@@ -58,8 +58,7 @@ bool if_status_handle_claims(struct if_status_mgr *mgr,
                              struct local_binding_data *binding_data,
                              const struct sbrec_chassis *chassis_rec,
                              struct hmap *tracked_datapath,
-                             const struct sbrec_port_binding_table *pb_table,
-                             bool sb_readonly);
+                             const struct sbrec_port_binding_table *pb_table);
 void if_status_mgr_remove_ovn_installed(struct if_status_mgr *mgr,
                                     const struct ovsrec_interface *iface_rec);
 uint16_t if_status_mgr_iface_get_mtu(const struct if_status_mgr *mgr,
@@ -68,5 +67,7 @@ bool if_status_mgr_iface_update(const struct if_status_mgr *mgr,
                                 const struct ovsrec_interface *iface_rec);
 bool if_status_is_port_claimed(const struct if_status_mgr *mgr,
                                const char *iface_id);
+bool if_status_reclaimed(struct if_status_mgr *mgr, const char *iface_id);
+struct sset * get_claimed_cr(struct if_status_mgr *mgr);
 
 # endif /* controller/if-status.h */
