@@ -67,17 +67,18 @@ struct uuid;
 
 /* Start of LOG_PIPELINE_LEN tables. */
 #define OFTABLE_LOG_INGRESS_PIPELINE      8
-#define OFTABLE_OUTPUT_LARGE_PKT_DETECT  40
-#define OFTABLE_OUTPUT_LARGE_PKT_PROCESS 41
-#define OFTABLE_REMOTE_OUTPUT            42
-#define OFTABLE_LOCAL_OUTPUT             43
-#define OFTABLE_CHECK_LOOPBACK           44
+#define OFTABLE_OUTPUT_LARGE_PKT_DETECT  41
+#define OFTABLE_OUTPUT_LARGE_PKT_PROCESS 42
+#define OFTABLE_REMOTE_OUTPUT            43
+#define OFTABLE_REMOTE_VTEP_OUTPUT       44
+#define OFTABLE_LOCAL_OUTPUT             45
+#define OFTABLE_CHECK_LOOPBACK           46
 
 /* Start of the OUTPUT section of the pipeline. */
 #define OFTABLE_OUTPUT_INIT OFTABLE_OUTPUT_LARGE_PKT_DETECT
 
 /* Start of LOG_PIPELINE_LEN tables. */
-#define OFTABLE_LOG_EGRESS_PIPELINE      45
+#define OFTABLE_LOG_EGRESS_PIPELINE      47
 #define OFTABLE_SAVE_INPORT              64
 #define OFTABLE_LOG_TO_PHY               65
 #define OFTABLE_MAC_BINDING              66
@@ -100,6 +101,8 @@ struct uuid;
 #define OFTABLE_CT_ORIG_TP_DST_LOAD      83
 #define OFTABLE_FLOOD_REMOTE_CHASSIS     84
 #define OFTABLE_CT_STATE_SAVE            85
+#define OFTABLE_CT_ORIG_PROTO_LOAD       86
+#define OFTABLE_GET_REMOTE_FDB           87
 
 /* Common defines shared between some controller components. */
 #define CHASSIS_FLOOD_INDEX_START 0x8000
@@ -170,7 +173,7 @@ bool lflow_handle_addr_set_update(const char *as_name, struct addr_set_diff *,
                                   struct lflow_ctx_out *,
                                   bool *changed);
 bool lflow_handle_changed_ref(enum objdep_type, const char *res_name,
-                              struct ovs_list *objs_todo,
+                              struct uuidset *objs_todo,
                               const void *in_arg, void *out_arg);
 
 void lflow_handle_changed_mac_bindings(
