@@ -60,7 +60,6 @@ struct physical_ctx {
     const struct sset *active_tunnels;
     const struct if_status_mgr *if_mgr;
     struct hmap *local_datapaths;
-    struct sset *local_lports;
     const struct shash *ct_zones;
     enum mf_field_id mff_ovn_geneve;
     struct shash *local_bindings;
@@ -70,6 +69,13 @@ struct physical_ctx {
     const char **encap_ips;
     struct physical_debug debug;
     bool always_tunnel;
+    const struct hmap *evpn_bindings;
+    const struct hmap *evpn_multicast_groups;
+    const struct hmap *evpn_fdbs;
+
+    /* Set of port binding names that have been already reprocessed during
+     * the I-P run. */
+    struct sset reprocessed_pbs;
 };
 
 void physical_register_ovs_idl(struct ovsdb_idl *);
@@ -81,4 +87,16 @@ bool physical_handle_flows_for_lport(const struct sbrec_port_binding *,
                                      bool removed,
                                      struct physical_ctx *,
                                      struct ovn_desired_flow_table *);
+void physical_multichassis_reprocess(const struct sbrec_port_binding *,
+                                     struct physical_ctx *,
+                                     struct ovn_desired_flow_table *);
+void physical_handle_evpn_binding_changes(
+    struct physical_ctx *, struct ovn_desired_flow_table *,
+    const struct hmapx *updated_bindings,
+    const struct hmapx *updated_multicast_groups,
+    const struct uuidset *removed_bindings,
+    const struct uuidset *removed_multicast_groups);
+void physical_handle_evpn_fdb_changes(struct ovn_desired_flow_table *,
+                                      const struct hmapx *updated_fdbs,
+                                      const struct uuidset *removed_fdbs);
 #endif /* controller/physical.h */
