@@ -31,11 +31,13 @@
 #include "ovn-util.h"
 #include "route-table.h"
 #include "route.h"
+#include "vec.h"
 
 #include "route-exchange-netlink.h"
 
 VLOG_DEFINE_THIS_MODULE(route_exchange_netlink);
 
+#define NETNL_REQ_BUFFER_SIZE 128
 #define TABLE_ID_VALID(table_id) (table_id != RT_TABLE_UNSPEC &&              \
                                   table_id != RT_TABLE_COMPAT &&              \
                                   table_id != RT_TABLE_DEFAULT &&             \
@@ -56,10 +58,12 @@ re_nl_create_vrf(const char *ifname, uint32_t table_id)
 
     size_t linkinfo_off, infodata_off;
     struct ifinfomsg *ifinfo;
-    struct ofpbuf request;
     int err;
 
-    ofpbuf_init(&request, 0);
+    struct ofpbuf request;
+    uint8_t request_stub[NETNL_REQ_BUFFER_SIZE];
+    ofpbuf_use_stub(&request, request_stub, sizeof(request_stub));
+
     nl_msg_put_nlmsghdr(&request, 0, RTM_NEWLINK,
                         NLM_F_REQUEST | NLM_F_ACK | NLM_F_CREATE | NLM_F_EXCL);
     ifinfo = ofpbuf_put_zeros(&request, sizeof *ifinfo);
@@ -83,10 +87,12 @@ int
 re_nl_delete_vrf(const char *ifname)
 {
     struct ifinfomsg *ifinfo;
-    struct ofpbuf request;
     int err;
 
-    ofpbuf_init(&request, 0);
+    struct ofpbuf request;
+    uint8_t request_stub[NETNL_REQ_BUFFER_SIZE];
+    ofpbuf_use_stub(&request, request_stub, sizeof(request_stub));
+
     nl_msg_put_nlmsghdr(&request, 0, RTM_DELLINK, NLM_F_REQUEST | NLM_F_ACK);
     ifinfo = ofpbuf_put_zeros(&request, sizeof *ifinfo);
     nl_msg_put_string(&request, IFLA_IFNAME, ifname);
@@ -103,13 +109,15 @@ modify_route(uint32_t type, uint32_t flags_arg, uint32_t table_id,
 {
     uint32_t flags = NLM_F_REQUEST | NLM_F_ACK;
     bool is_ipv4 = IN6_IS_ADDR_V4MAPPED(dst);
-    struct ofpbuf request;
     struct rtmsg *rt;
     int err;
 
     flags |= flags_arg;
 
-    ofpbuf_init(&request, 0);
+    struct ofpbuf request;
+    uint8_t request_stub[NETNL_REQ_BUFFER_SIZE];
+    ofpbuf_use_stub(&request, request_stub, sizeof(request_stub));
+
     nl_msg_put_nlmsghdr(&request, 0, type, flags);
     rt = ofpbuf_put_zeros(&request, sizeof *rt);
     rt->rtm_family = is_ipv4 ? AF_INET : AF_INET6;
@@ -188,20 +196,12 @@ re_nl_delete_route(uint32_t table_id, const struct in6_addr *dst,
     return modify_route(RTM_DELROUTE, 0, table_id, dst, plen, priority);
 }
 
-void
-re_nl_learned_routes_destroy(struct ovs_list *learned_routes)
-{
-    struct re_nl_received_route_node *rr;
-    LIST_FOR_EACH_POP (rr, list_node, learned_routes) {
-        free(rr);
-    }
-}
-
 struct route_msg_handle_data {
     const struct sbrec_datapath_binding *db;
     struct hmapx *routes_to_advertise;
-    struct ovs_list *learned_routes;
+    struct vector *learned_routes;
     const struct hmap *routes;
+    uint32_t table_id; /* requested table id. */
     int ret;
 };
 
@@ -213,6 +213,13 @@ handle_route_msg(const struct route_table_msg *msg, void *data)
     struct advertise_route_entry *ar;
     int err;
 
+    if (handle_data->table_id != rd->rta_table_id) {
+        /* We do not have the NLM_F_DUMP_FILTERED info here, so check if the
+         * reported table_id matches the requested one.
+         */
+        return;
+    }
+
     /* This route is not from us, so we learn it. */
     if (rd->rtm_protocol != RTPROT_OVN) {
         if (!handle_data->learned_routes) {
@@ -228,14 +235,17 @@ handle_route_msg(const struct route_table_msg *msg, void *data)
                  * As we just want to learn remote routes we do not need it.*/
                 continue;
             }
-            struct re_nl_received_route_node *rr = xmalloc(sizeof *rr);
-            rr->db = handle_data->db;
-            rr->prefix = rd->rta_dst;
-            rr->plen = rd->rtm_dst_len;
-            rr->nexthop = nexthop->addr;
-            memcpy(rr->ifname, nexthop->ifname, IFNAMSIZ);
-            rr->ifname[IFNAMSIZ] = 0;
-            ovs_list_push_back(handle_data->learned_routes, &rr->list_node);
+            struct re_nl_received_route_node rr;
+            rr = (struct re_nl_received_route_node) {
+                .db = handle_data->db,
+                .prefix = rd->rta_dst,
+                .plen = rd->rtm_dst_len,
+                .nexthop = nexthop->addr,
+            };
+            memcpy(rr.ifname, nexthop->ifname, IFNAMSIZ);
+            rr.ifname[IFNAMSIZ] = '\0';
+
+            vector_push(handle_data->learned_routes, &rr);
         }
         return;
     }
@@ -272,7 +282,7 @@ handle_route_msg(const struct route_table_msg *msg, void *data)
 
 int
 re_nl_sync_routes(uint32_t table_id, const struct hmap *routes,
-                  struct ovs_list *learned_routes,
+                  struct vector *learned_routes,
                   const struct sbrec_datapath_binding *db)
 {
     struct hmapx routes_to_advertise = HMAPX_INITIALIZER(&routes_to_advertise);
@@ -291,6 +301,7 @@ re_nl_sync_routes(uint32_t table_id, const struct hmap *routes,
         .routes_to_advertise = &routes_to_advertise,
         .learned_routes = learned_routes,
         .db = db,
+        .table_id = table_id,
     };
     route_table_dump_one_table(table_id, handle_route_msg, &data);
     ret = data.ret;
@@ -331,6 +342,7 @@ re_nl_cleanup_routes(uint32_t table_id)
     struct route_msg_handle_data data = {
         .routes_to_advertise = NULL,
         .learned_routes = NULL,
+        .table_id = table_id,
     };
     route_table_dump_one_table(table_id, handle_route_msg, &data);
 
