@@ -19,22 +19,21 @@
 #define ROUTE_EXCHANGE_NETLINK_H 1
 
 #include <stdint.h>
-#include "openvswitch/list.h"
 #include <netinet/in.h>
 #include <net/if.h>
 
-/* This value is arbitrary but currently unused.
- * See the kernel rtnetlink UAPI at
- * https://github.com/torvalds/linux/blob/master/include/uapi/linux/rtnetlink.h
- * */
+/* The following definition should be available in Linux 6.15 and might be
+ * missing if we have older headers. */
+#ifndef RTPROT_OVN
 #define RTPROT_OVN 84
+#endif
 
 struct in6_addr;
 struct hmap;
+struct vector;
 
 struct re_nl_received_route_node {
     const struct sbrec_datapath_binding *db;
-    struct ovs_list list_node;
     struct in6_addr prefix;
     unsigned int plen;
     struct in6_addr nexthop;
@@ -50,9 +49,8 @@ int re_nl_add_route(uint32_t table_id, const struct in6_addr *dst,
 int re_nl_delete_route(uint32_t table_id, const struct in6_addr *dst,
                        unsigned int plen, unsigned int priority);
 
-void re_nl_learned_routes_destroy(struct ovs_list *learned_routes);
 int re_nl_sync_routes(uint32_t table_id, const struct hmap *routes,
-                      struct ovs_list *learned_routes,
+                      struct vector *learned_routes,
                       const struct sbrec_datapath_binding *db);
 
 int re_nl_cleanup_routes(uint32_t table_id);
