@@ -52,7 +52,6 @@ advertise_route_hash(const struct in6_addr *dst, unsigned int plen)
 const struct sbrec_port_binding*
 route_exchange_find_port(struct ovsdb_idl_index *sbrec_port_binding_by_name,
                          const struct sbrec_chassis *chassis,
-                         const struct sset *active_tunnels,
                          const struct sbrec_port_binding *pb)
 {
     if (!pb) {
@@ -69,7 +68,7 @@ route_exchange_find_port(struct ovsdb_idl_index *sbrec_port_binding_by_name,
         return NULL;
     }
 
-    if (!lport_pb_is_chassis_resident(chassis, active_tunnels, cr_pb)) {
+    if (!lport_pb_is_chassis_resident(chassis, cr_pb)) {
         return NULL;
     }
 
@@ -158,20 +157,19 @@ route_run(struct route_ctx_in *r_ctx_in,
     build_port_mapping(&port_mapping, r_ctx_in->dynamic_routing_port_mapping);
 
     HMAP_FOR_EACH (ld, hmap_node, r_ctx_in->local_datapaths) {
-        if (!ld->n_peer_ports || ld->is_switch) {
+        if (vector_is_empty(&ld->peer_ports) || ld->is_switch) {
             continue;
         }
         ad = NULL;
 
         /* This is a LR datapath, find LRPs with route exchange options
          * that are bound locally. */
-        for (size_t i = 0; i < ld->n_peer_ports; i++) {
-            const struct sbrec_port_binding *local_peer
-                = ld->peer_ports[i].local;
+        const struct peer_ports *peers;
+        VECTOR_FOR_EACH_PTR (&ld->peer_ports, peers) {
+            const struct sbrec_port_binding *local_peer = peers->local;
             const struct sbrec_port_binding *repb =
                 route_exchange_find_port(r_ctx_in->sbrec_port_binding_by_name,
                                          r_ctx_in->chassis,
-                                         r_ctx_in->active_tunnels,
                                          local_peer);
             if (!repb) {
                 continue;
@@ -255,9 +253,12 @@ route_run(struct route_ctx_in *r_ctx_in,
 
         unsigned int priority = PRIORITY_DEFAULT;
         if (route->tracked_port) {
+            bool redistribute_local_bound_only =
+                smap_get_bool(&route->logical_port->options,
+                              "dynamic-routing-redistribute-local-only",
+                              false);
             if (lport_is_local(r_ctx_in->sbrec_port_binding_by_name,
                                r_ctx_in->chassis,
-                               r_ctx_in->active_tunnels,
                                route->tracked_port->logical_port)) {
                 priority = PRIORITY_LOCAL_BOUND;
                 sset_add(r_ctx_out->tracked_ports_local,
@@ -265,6 +266,11 @@ route_run(struct route_ctx_in *r_ctx_in,
             } else {
                 sset_add(r_ctx_out->tracked_ports_remote,
                          route->tracked_port->logical_port);
+                if (redistribute_local_bound_only) {
+                    /* We're not advertising routes whose 'tracked_port' is
+                     * not local, skip this route. */
+                    continue;
+                }
             }
         }
 
