@@ -121,6 +121,7 @@ struct collector_set_ids;
     OVNACT(SCTP_ABORT,        ovnact_nest)            \
     OVNACT(PUT_FDB,           ovnact_put_fdb)         \
     OVNACT(GET_FDB,           ovnact_get_fdb)         \
+    OVNACT(GET_REMOTE_FDB,    ovnact_get_fdb)         \
     OVNACT(LOOKUP_FDB,        ovnact_lookup_fdb)      \
     OVNACT(CHECK_IN_PORT_SEC,  ovnact_result)         \
     OVNACT(CHECK_OUT_PORT_SEC, ovnact_result)         \
@@ -134,8 +135,10 @@ struct collector_set_ids;
     OVNACT(CT_ORIG_NW_DST,    ovnact_result)          \
     OVNACT(CT_ORIG_IP6_DST,   ovnact_result)          \
     OVNACT(CT_ORIG_TP_DST,    ovnact_result)          \
+    OVNACT(CT_ORIG_PROTO,     ovnact_result)          \
     OVNACT(FLOOD_REMOTE,      ovnact_null)            \
-    OVNACT(CT_STATE_SAVE,     ovnact_result)
+    OVNACT(CT_STATE_SAVE,     ovnact_result)          \
+    OVNACT(MIRROR,            ovnact_mirror)          \
 
 /* enum ovnact_type, with a member OVNACT_<ENUM> for each action. */
 enum OVS_PACKED_ENUM ovnact_type {
@@ -422,7 +425,7 @@ struct ovnact_set_queue {
 };
 
 /* OVNACT_DNS_LOOKUP, OVNACT_CHK_LB_HAIRPIN, OVNACT_CHK_LB_HAIRPIN_REPLY,
- * OVNACT_CT_ORIG_NW_DST, CT_ORIG_IP6_DST, CT_ORIG_TP_DST */
+ * OVNACT_CT_ORIG_NW_DST, CT_ORIG_IP6_DST, CT_ORIG_TP_DST, CT_ORIG_PROTO */
 struct ovnact_result {
     struct ovnact ovnact;
     struct expr_field dst;      /* destination field. */
@@ -538,6 +541,15 @@ struct ovnact_commit_lb_aff {
     uint16_t timeout;
 };
 
+/* OVNACT_MIRROR. */
+struct ovnact_mirror {
+    struct ovnact ovnact;
+
+    /* Argument. */
+    char *port;     /* Mirror serving port for output
+                       action. */
+};
+
 #define OVN_FIELD_NOTE_MAGIC "ovn"
 
 struct ovn_field_note_header {
@@ -789,8 +801,8 @@ OVNACTS
     /* put_fdb(inport, eth.src). */                                           \
     ACTION_OPCODE(PUT_FDB)                                                    \
                                                                               \
-    /* activation_strategy_rarp() */                                          \
-    ACTION_OPCODE(ACTIVATION_STRATEGY_RARP)                                   \
+    /* activation_strategy() */                                               \
+    ACTION_OPCODE(ACTIVATION_STRATEGY)                                        \
                                                                               \
     /* split buffer action. */                                                \
     ACTION_OPCODE(SPLIT_BUF_ACTION)                                           \
@@ -945,6 +957,8 @@ struct ovnact_encode_params {
                                   * 'ct_snat_to_vip' to resubmit. */
     uint8_t fdb_ptable; /* OpenFlow table for
                          * 'get_fdb' to resubmit. */
+    uint8_t remote_fdb_ptable; /* OpenFlow table for
+                                * 'get_remote_fdb' to resubmit. */
     uint8_t fdb_lookup_ptable; /* OpenFlow table for
                                 * 'lookup_fdb' to resubmit. */
     uint8_t in_port_sec_ptable; /* OpenFlow table for
@@ -963,6 +977,8 @@ struct ovnact_encode_params {
                                      *  to resubmit. */
     uint32_t ct_tp_dst_load_table; /* OpenFlow table for 'ct_tp_dst'
                                     *  to resubmit. */
+    uint32_t ct_proto_load_table; /* OpenFlow table for 'ct_proto'
+                                   *  to resubmit. */
     uint32_t flood_remote_table; /* OpenFlow table for 'chassis_flood'
                                   * to resubmit. */
     uint32_t ct_state_save_table; /* OpenFlow table for saving ct_state to
