@@ -60,11 +60,9 @@ enum ovs_feature_value {
 void ovs_feature_support_destroy(void);
 bool ovs_feature_is_supported(enum ovs_feature_value feature);
 bool ovs_feature_support_run(const struct smap *ovs_capabilities,
-                             const char *conn_target, int probe_interval,
-                             const char *db_target);
+                             const char *conn_target, int probe_interval);
 bool ovs_feature_set_discovered(void);
 uint32_t ovs_feature_max_meters_get(void);
 uint32_t ovs_feature_max_select_groups_get(void);
-size_t ovs_features_max_flow_table_prefixes_get(void);
 
 #endif
