@@ -55,16 +55,16 @@ enum ovn_controller_event {
 
 #define MFF_N_LOG_REGS 10
 
-#define MFF_LOG_LB_AFF_MATCH_IP4_ADDR       MFF_REG4
-#define MFF_LOG_LB_AFF_MATCH_IP6_ADDR       MFF_XXREG1
-#define MFF_LOG_LB_AFF_MATCH_PORT           MFF_REG2
+#define MFF_LOG_LB_AFF_MATCH_IP4_ADDR MFF_REG4
+#define MFF_LOG_LB_AFF_MATCH_IP6_ADDR MFF_XXREG1
+#define MFF_LOG_LB_AFF_MATCH_PORT     MFF_REG2
 
-#define MFF_LOG_CT_ORIG_NW_DST_ADDR         MFF_REG1   /* REG_ORIG_DIP_IPV4 */
-#define MFF_LOG_CT_ORIG_IP6_DST_ADDR        MFF_XXREG1 /* REG_ORIG_DIP_IPV6 */
-#define MFF_LOG_CT_ORIG_TP_DST_PORT         MFF_REG2   /* REG_ORIG_TP_DPORT
-                                                        * (bits 0..15). */
-#define MFF_LOG_CT_SAVED_STATE              MFF_REG4   /* REG_CT_STATE
-                                                        * (bits 0..8). */
+#define MFF_LOG_RESULT_REG            MFF_XXREG1
+
+#define MFF_LOG_CT_SAVED_STATE        MFF_REG4
+
+#define MFF_LOG_REMOTE_OUTPORT MFF_REG1  /* Logical remote output
+                                          * port (32 bits). */
 
 /* Logical registers that are needed for backwards
  * compatibility with older northd versions.
@@ -103,6 +103,7 @@ enum mff_log_flags_bits {
     MLF_FROM_CTRL_BIT = 19,
     MLF_UNSNAT_NEW_BIT = 20,
     MLF_UNSNAT_NOT_TRACKED_BIT = 21,
+    MLF_IGMP_IGMP_SNOOP_INJECT_BIT = 22,
     MLF_NETWORK_ID_START_BIT = 28,
     MLF_NETWORK_ID_END_BIT = 31,
 };
@@ -114,7 +115,7 @@ enum mff_log_flags {
 
     /* Indicate that a packet was received from a ramp switch to compensate for
      * the lack of egress port information available in ramp switch
-     * encapsulation.  Egress port information is available for Geneve, STT and
+     * encapsulation.  Egress port information is available for Geneve and
      * regular VXLAN tunnel types. */
     MLF_RCV_FROM_RAMP = (1 << MLF_RCV_FROM_RAMP_BIT),
 
@@ -169,6 +170,9 @@ enum mff_log_flags {
     /* Indicate that the packet didn't go through unSNAT. */
     MLF_UNSNAT_NOT_TRACKED = (1 << MLF_UNSNAT_NOT_TRACKED_BIT),
 
+    /* Indicate that this is an IGMP packet reinjected by ovn-controller. */
+    MLF_IGMP_IGMP_SNOOP = (1 << MLF_IGMP_IGMP_SNOOP_INJECT_BIT),
+
     /* Assign network ID to packet to choose correct network for snat when
      * lb_force_snat_ip=router_ip. */
     MLF_NETWORK_ID = (OVN_MAX_NETWORK_ID << MLF_NETWORK_ID_START_BIT),
