@@ -30,6 +30,7 @@
 #include "inc-proc-eng.h"
 #include "timeval.h"
 #include "unixctl.h"
+#include "vec.h"
 
 VLOG_DEFINE_THIS_MODULE(inc_proc_eng);
 
@@ -37,8 +38,8 @@ static bool engine_force_recompute = false;
 static bool engine_run_canceled = false;
 static const struct engine_context *engine_context;
 
-static struct engine_node **engine_nodes;
-static size_t engine_n_nodes;
+static struct vector engine_nodes =
+    VECTOR_EMPTY_INITIALIZER(struct engine_node *);
 
 static const char *engine_node_state_name[EN_STATE_MAX] = {
     [EN_STALE]     = "Stale",
@@ -93,50 +94,32 @@ engine_set_context(const struct engine_context *ctx)
 /* Builds the topologically sorted 'sorted_nodes' array starting from
  * 'node'.
  */
-static struct engine_node **
-engine_topo_sort(struct engine_node *node, struct engine_node **sorted_nodes,
-                 size_t *n_count, size_t *n_size)
+static void
+engine_topo_sort(struct engine_node *node, struct vector *sorted_nodes)
 {
     /* It's not so efficient to walk the array of already sorted nodes but
      * we know that sorting is done only once at startup so it's ok for now.
      */
-    for (size_t i = 0; i < *n_count; i++) {
-        if (sorted_nodes[i] == node) {
-            return sorted_nodes;
+    struct engine_node *sorted_node;
+    VECTOR_FOR_EACH (sorted_nodes, sorted_node) {
+        if (sorted_node == node) {
+            return;
         }
     }
 
     for (size_t i = 0; i < node->n_inputs; i++) {
-        sorted_nodes = engine_topo_sort(node->inputs[i].node, sorted_nodes,
-                                        n_count, n_size);
+        engine_topo_sort(node->inputs[i].node, sorted_nodes);
     }
-    if (*n_count == *n_size) {
-        sorted_nodes = x2nrealloc(sorted_nodes, n_size, sizeof *sorted_nodes);
-    }
-    sorted_nodes[(*n_count)] = node;
-    (*n_count)++;
-    return sorted_nodes;
-}
 
-/* Return the array of topologically sorted nodes when starting from
- * 'node'. Stores the number of nodes in 'n_count'.
- */
-static struct engine_node **
-engine_get_nodes(struct engine_node *node, size_t *n_count)
-{
-    size_t n_size = 0;
-
-    *n_count = 0;
-    return engine_topo_sort(node, NULL, n_count, &n_size);
+    vector_push(sorted_nodes, &node);
 }
 
 static void
 engine_clear_stats(struct unixctl_conn *conn, int argc OVS_UNUSED,
                    const char *argv[] OVS_UNUSED, void *arg OVS_UNUSED)
 {
-    for (size_t i = 0; i < engine_n_nodes; i++) {
-        struct engine_node *node = engine_nodes[i];
-
+    struct engine_node *node;
+    VECTOR_FOR_EACH (&engine_nodes, node) {
         memset(&node->stats, 0, sizeof node->stats);
     }
     unixctl_command_reply(conn, NULL);
@@ -150,9 +133,8 @@ engine_dump_stats(struct unixctl_conn *conn, int argc,
     const char *dump_eng_node_name = (argc > 1 ? argv[1] : NULL);
     const char *dump_stat_type = (argc > 2 ? argv[2] : NULL);
 
-    for (size_t i = 0; i < engine_n_nodes; i++) {
-        struct engine_node *node = engine_nodes[i];
-
+    struct engine_node *node;
+    VECTOR_FOR_EACH (&engine_nodes, node) {
         if (dump_eng_node_name && strcmp(node->name, dump_eng_node_name)) {
             continue;
         }
@@ -210,17 +192,28 @@ engine_set_log_timeout_cmd(struct unixctl_conn *conn, int argc OVS_UNUSED,
     unixctl_command_reply(conn, NULL);
 }
 
+static void
+engine_get_compute_failure_info(struct engine_node *node)
+{
+    VLOG_DBG("Node \"%s\" is missing compute failure debug info.", node->name);
+}
+
 void
 engine_init(struct engine_node *node, struct engine_arg *arg)
 {
-    engine_nodes = engine_get_nodes(node, &engine_n_nodes);
+    engine_topo_sort(node, &engine_nodes);
 
-    for (size_t i = 0; i < engine_n_nodes; i++) {
-        if (engine_nodes[i]->init) {
-            engine_nodes[i]->data =
-                engine_nodes[i]->init(engine_nodes[i], arg);
+    struct engine_node *sorted_node;
+    VECTOR_FOR_EACH (&engine_nodes, sorted_node) {
+        if (sorted_node->init) {
+            sorted_node->data = sorted_node->init(sorted_node, arg);
         } else {
-            engine_nodes[i]->data = NULL;
+            sorted_node->data = NULL;
+        }
+        if (!sorted_node->get_compute_failure_info) {
+            /* Provide default get_compute_failure_info implementation. */
+            sorted_node->get_compute_failure_info =
+                engine_get_compute_failure_info;
         }
     }
 
@@ -237,19 +230,18 @@ engine_init(struct engine_node *node, struct engine_arg *arg)
 void
 engine_cleanup(void)
 {
-    for (size_t i = 0; i < engine_n_nodes; i++) {
-        if (engine_nodes[i]->clear_tracked_data) {
-            engine_nodes[i]->clear_tracked_data(engine_nodes[i]->data);
+    struct engine_node *node;
+    VECTOR_FOR_EACH (&engine_nodes, node) {
+        if (node->clear_tracked_data) {
+            node->clear_tracked_data(node->data);
         }
 
-        if (engine_nodes[i]->cleanup) {
-            engine_nodes[i]->cleanup(engine_nodes[i]->data);
+        if (node->cleanup) {
+            node->cleanup(node->data);
         }
-        free(engine_nodes[i]->data);
+        free(node->data);
     }
-    free(engine_nodes);
-    engine_nodes = NULL;
-    engine_n_nodes = 0;
+    vector_destroy(&engine_nodes);
 }
 
 struct engine_node *
@@ -274,7 +266,8 @@ engine_get_input_data(const char *input_name, struct engine_node *node)
 
 void
 engine_add_input(struct engine_node *node, struct engine_node *input,
-                 bool (*change_handler)(struct engine_node *, void *))
+                 enum engine_input_handler_result (*change_handler)
+                     (struct engine_node *, void *))
 {
     ovs_assert(node->n_inputs < ENGINE_MAX_INPUT);
     node->inputs[node->n_inputs].node = input;
@@ -282,6 +275,17 @@ engine_add_input(struct engine_node *node, struct engine_node *input,
     node->n_inputs ++;
 }
 
+void
+engine_add_input_with_compute_debug(
+        struct engine_node *node, struct engine_node *input,
+        enum engine_input_handler_result (*change_handler)
+            (struct engine_node *, void *),
+        void (*get_compute_failure_info)(struct engine_node *))
+{
+    engine_add_input(node, input, change_handler);
+    node->get_compute_failure_info = get_compute_failure_info;
+}
+
 struct ovsdb_idl_index *
 engine_ovsdb_node_get_index(struct engine_node *node, const char *name)
 {
@@ -307,19 +311,25 @@ engine_ovsdb_node_add_index(struct engine_node *node, const char *name,
     ed->n_indexes ++;
 }
 
-void
-engine_set_node_state_at(struct engine_node *node,
-                         enum engine_node_state state,
-                         const char *where)
+static void
+engine_set_node_state(struct engine_node *node,
+                      enum engine_node_state state,
+                      const char *reason_fmt, ...)
 {
     if (node->state == state) {
         return;
     }
 
-    VLOG_DBG("%s: node: %s, old_state %s, new_state %s",
-             where, node->name,
-             engine_node_state_name[node->state],
-             engine_node_state_name[state]);
+    if (VLOG_IS_DBG_ENABLED()) {
+        va_list args;
+        va_start(args, reason_fmt);
+        char *reason = xvasprintf(reason_fmt, args);
+        VLOG_DBG("node: %s, old_state %s, new_state %s, reason: %s.",
+                 node->name, engine_node_state_name[node->state],
+                 engine_node_state_name[state], reason);
+        va_end(args);
+        free(reason);
+    }
 
     node->state = state;
 }
@@ -346,8 +356,9 @@ engine_node_changed(struct engine_node *node)
 bool
 engine_has_run(void)
 {
-    for (size_t i = 0; i < engine_n_nodes; i++) {
-        if (engine_nodes[i]->state != EN_STALE) {
+    struct engine_node *node;
+    VECTOR_FOR_EACH (&engine_nodes, node) {
+        if (node->state != EN_STALE) {
             return true;
         }
     }
@@ -357,8 +368,9 @@ engine_has_run(void)
 bool
 engine_has_updated(void)
 {
-    for (size_t i = 0; i < engine_n_nodes; i++) {
-        if (engine_nodes[i]->state == EN_UPDATED) {
+    struct engine_node *node;
+    VECTOR_FOR_EACH (&engine_nodes, node) {
+        if (node->state == EN_UPDATED) {
             return true;
         }
     }
@@ -390,11 +402,12 @@ void
 engine_init_run(void)
 {
     VLOG_DBG("Initializing new run");
-    for (size_t i = 0; i < engine_n_nodes; i++) {
-        engine_set_node_state(engine_nodes[i], EN_STALE);
+    struct engine_node *node;
+    VECTOR_FOR_EACH (&engine_nodes, node) {
+        engine_set_node_state(node, EN_STALE, "engine_init_run");
 
-        if (engine_nodes[i]->clear_tracked_data) {
-            engine_nodes[i]->clear_tracked_data(engine_nodes[i]->data);
+        if (node->clear_tracked_data) {
+            node->clear_tracked_data(node->data);
         }
     }
 }
@@ -415,7 +428,7 @@ engine_recompute(struct engine_node *node, bool allowed,
 
     if (!allowed) {
         VLOG_DBG("node: %s, recompute (%s) canceled", node->name, reason);
-        engine_set_node_state(node, EN_CANCELED);
+        engine_set_node_state(node, EN_CANCELED, "recompute not allowed");
         goto done;
     }
 
@@ -427,7 +440,8 @@ engine_recompute(struct engine_node *node, bool allowed,
 
     /* Run the node handler which might change state. */
     long long int now = time_msec();
-    node->run(node, node->data);
+    engine_set_node_state(node, node->run(node, node->data),
+                          "recompute run() result");
     node->stats.recompute++;
     long long int delta_time = time_msec() - now;
     if (delta_time > engine_compute_log_timeout_msec) {
@@ -447,29 +461,39 @@ static bool
 engine_compute(struct engine_node *node, bool recompute_allowed)
 {
     for (size_t i = 0; i < node->n_inputs; i++) {
+        struct engine_node *input_node = node->inputs[i].node;
+
         /* If the input node data changed call its change handler. */
-        if (node->inputs[i].node->state == EN_UPDATED) {
+        if (input_node->state == EN_UPDATED) {
             /* If the input change can't be handled incrementally, run
              * the node handler.
              */
             long long int now = time_msec();
-            bool handled = node->inputs[i].change_handler(node, node->data);
+            enum engine_input_handler_result handled;
+            handled = node->inputs[i].change_handler(node, node->data);
             long long int delta_time = time_msec() - now;
             if (delta_time > engine_compute_log_timeout_msec) {
                 static struct vlog_rate_limit rl =
                     VLOG_RATE_LIMIT_INIT(20, 10);
                 VLOG_INFO_RL(&rl, "node: %s, handler for input %s took %lldms",
-                             node->name, node->inputs[i].node->name,
-                             delta_time);
+                             node->name, input_node->name, delta_time);
             } else {
                 VLOG_DBG("node: %s, handler for input %s took %lldms",
-                         node->name, node->inputs[i].node->name, delta_time);
+                         node->name, input_node->name, delta_time);
             }
-            if (!handled) {
+            if (handled == EN_UNHANDLED) {
+                input_node->get_compute_failure_info(input_node);
                 engine_recompute(node, recompute_allowed,
                                  "failed handler for input %s",
-                                 node->inputs[i].node->name);
+                                 input_node->name);
                 return (node->state != EN_CANCELED);
+            } else if (!engine_node_changed(node)) {
+                /* We only want to update the state if the node is unchanged.
+                 * Otherwise, handlers might change the state from EN_UPDATED
+                 * back to EN_UNCHANGED.
+                 */
+                engine_set_node_state(node, (enum engine_node_state) handled,
+                                      "input %s updated", input_node->name);
             }
         }
     }
@@ -483,7 +507,8 @@ engine_run_node(struct engine_node *node, bool recompute_allowed)
 {
     if (!node->n_inputs) {
         /* Run the node handler which might change state. */
-        node->run(node, node->data);
+        engine_set_node_state(node, node->run(node, node->data),
+                              "run() result due to having no inputs");
         node->stats.recompute++;
         return;
     }
@@ -498,14 +523,16 @@ engine_run_node(struct engine_node *node, bool recompute_allowed)
      */
     bool need_compute = false;
     for (size_t i = 0; i < node->n_inputs; i++) {
-        if (node->inputs[i].node->state == EN_UPDATED) {
+        struct engine_node *input_node = node->inputs[i].node;
+        if (input_node->state == EN_UPDATED) {
             need_compute = true;
 
             /* Trigger a recompute if we don't have a change handler. */
             if (!node->inputs[i].change_handler) {
                 engine_recompute(node, recompute_allowed,
                                  "missing handler for input %s",
-                                 node->inputs[i].node->name);
+                                 input_node->name);
+                input_node->get_compute_failure_info(input_node);
                 return;
             }
         }
@@ -524,7 +551,7 @@ engine_run_node(struct engine_node *node, bool recompute_allowed)
      * still valid.
      */
     if (!engine_node_changed(node)) {
-        engine_set_node_state(node, EN_UNCHANGED);
+        engine_set_node_state(node, EN_UNCHANGED, "no change detected");
     }
 }
 
@@ -539,11 +566,12 @@ engine_run(bool recompute_allowed)
     }
 
     engine_run_canceled = false;
-    for (size_t i = 0; i < engine_n_nodes; i++) {
-        engine_run_node(engine_nodes[i], recompute_allowed);
+    struct engine_node *node;
+    VECTOR_FOR_EACH (&engine_nodes, node) {
+        engine_run_node(node, recompute_allowed);
 
-        if (engine_nodes[i]->state == EN_CANCELED) {
-            engine_nodes[i]->stats.cancel++;
+        if (node->state == EN_CANCELED) {
+            node->stats.cancel++;
             engine_run_canceled = true;
             return;
         }
@@ -553,17 +581,19 @@ engine_run(bool recompute_allowed)
 bool
 engine_need_run(void)
 {
-    for (size_t i = 0; i < engine_n_nodes; i++) {
+    struct engine_node *node;
+    VECTOR_FOR_EACH (&engine_nodes, node) {
         /* Check only leaf nodes for updates. */
-        if (engine_nodes[i]->n_inputs) {
+        if (node->n_inputs) {
             continue;
         }
 
-        engine_nodes[i]->run(engine_nodes[i], engine_nodes[i]->data);
-        engine_nodes[i]->stats.recompute++;
-        VLOG_DBG("input node: %s, state: %s", engine_nodes[i]->name,
-                 engine_node_state_name[engine_nodes[i]->state]);
-        if (engine_nodes[i]->state == EN_UPDATED) {
+        engine_set_node_state(node, node->run(node, node->data),
+                              "checking if engine needs to be run");
+        node->stats.recompute++;
+        VLOG_DBG("input node: %s, state: %s", node->name,
+                 engine_node_state_name[node->state]);
+        if (node->state == EN_UPDATED) {
             return true;
         }
     }
