@@ -170,22 +170,6 @@ struct engine_arg {
 
 struct engine_node;
 
-struct engine_node_input {
-    /* The input node. */
-    struct engine_node *node;
-
-    /* Change handler for changes of the input node. The changes may need to be
-     * evaluated against all the other inputs. Returns:
-     *  - true: if change can be handled
-     *  - false: if change cannot be handled (indicating full recompute needed)
-     * A change handler can also call engine_get_context() but it must make
-     * sure the txn pointers returned by it are non-NULL. In case the change
-     * handler needs to use the txn pointers returned by engine_get_context(),
-     * and the pointers are NULL, the change handler MUST return false.
-     */
-    bool (*change_handler)(struct engine_node *node, void *data);
-};
-
 enum engine_node_state {
     EN_STALE,     /* Data in the node is not up to date with the DB. */
     EN_UPDATED,   /* Data in the node is valid but was updated during the
@@ -200,6 +184,33 @@ enum engine_node_state {
     EN_STATE_MAX,
 };
 
+enum engine_input_handler_result {
+    EN_UNHANDLED = -1,
+    EN_HANDLED_UPDATED = EN_UPDATED,
+    EN_HANDLED_UNCHANGED = EN_UNCHANGED,
+};
+
+struct engine_node_input {
+    /* The input node. */
+    struct engine_node *node;
+
+    /* Change handler for changes of the input node. The changes may need to be
+     * evaluated against all the other inputs. Returns:
+     *  - EN_UNHANDLED: the change cannot be handled (indicating full
+     *    recompute needed).
+     *  - EN_HANDLED_UPDATED: the change can be handled, and the node's
+     *    data was updated as a result.
+     *  - EN_HANDLED_UNCHANGED: the change can be handled, and the node's
+     *    data was left unchanged.
+     * A change handler can also call engine_get_context() but it must make
+     * sure the txn pointers returned by it are non-NULL. In case the change
+     * handler needs to use the txn pointers returned by engine_get_context(),
+     * and the pointers are NULL, the change handler MUST return EN_UNHANDLED.
+     */
+    enum engine_input_handler_result (*change_handler)
+        (struct engine_node *node, void *data);
+};
+
 struct engine_stats {
     uint64_t recompute;
     uint64_t compute;
@@ -241,7 +252,7 @@ struct engine_node {
      * implementation guarantees that the txn pointers returned
      * engine_get_context() are not NULL and valid.
      */
-    void (*run)(struct engine_node *node, void *data);
+    enum engine_node_state (*run)(struct engine_node *node, void *data);
 
     /* Method to validate if the 'internal_data' is valid. This allows users
      * to customize when 'data' can be used (e.g., even if the node
@@ -254,6 +265,11 @@ struct engine_node {
      * engine 'data'. It may be NULL. */
     void (*clear_tracked_data)(void *tracked_data);
 
+    /* Method used to dump info about node input compute failues. It may be
+     * NULL.
+     */
+    void (*get_compute_failure_info)(struct engine_node *);
+
     /* Engine stats. */
     struct engine_stats stats;
 };
@@ -295,7 +311,13 @@ void *engine_get_input_data(const char *input_name, struct engine_node *);
  * be able to process the change incrementally, and will fall back to call
  * the run method to recompute. */
 void engine_add_input(struct engine_node *node, struct engine_node *input,
-                      bool (*change_handler)(struct engine_node *, void *));
+                      enum engine_input_handler_result (*change_handler)
+                          (struct engine_node *, void *));
+void engine_add_input_with_compute_debug(
+        struct engine_node *node, struct engine_node *input,
+        enum engine_input_handler_result (*change_handler)
+            (struct engine_node *, void *),
+        void (*get_compute_failure_info)(struct engine_node *));
 
 /* Force the engine to recompute everything. It is used
  * in circumstances when we are not sure there is change or not, or
@@ -324,10 +346,6 @@ const struct engine_context *engine_get_context(void);
 
 void engine_set_context(const struct engine_context *);
 
-void engine_set_node_state_at(struct engine_node *node,
-                              enum engine_node_state state,
-                              const char *where);
-
 /* Return true if during the last iteration the node's data was updated. */
 bool engine_node_changed(struct engine_node *node);
 
@@ -360,10 +378,6 @@ void *engine_get_data(struct engine_node *node);
  */
 void *engine_get_internal_data(struct engine_node *node);
 
-/* Set the state of the node and log changes. */
-#define engine_set_node_state(node, state) \
-    engine_set_node_state_at(node, state, OVS_SOURCE_LOCATOR)
-
 /* Trigger a full recompute. */
 void engine_trigger_recompute(void);
 
@@ -385,10 +399,10 @@ struct ovsdb_idl_index * engine_ovsdb_node_get_index(struct engine_node *,
                                                      const char *name);
 
 /* Any engine node can use this function for no-op handlers. */
-static inline bool
+static inline enum engine_input_handler_result
 engine_noop_handler(struct engine_node *node OVS_UNUSED, void *data OVS_UNUSED)
 {
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
 /* Adds an OVSDB IDL index to the node. This should be called only after
@@ -397,44 +411,56 @@ engine_noop_handler(struct engine_node *node OVS_UNUSED, void *data OVS_UNUSED)
 void engine_ovsdb_node_add_index(struct engine_node *, const char *name,
                                  struct ovsdb_idl_index *);
 
-/* Macro to define an engine node. */
-#define ENGINE_NODE_DEF(NAME, NAME_STR, CLEAR_TRACKED_DATA, IS_VALID) \
+#define ENGINE_NODE_DEF_START(NAME, NAME_STR) \
     struct engine_node en_##NAME = { \
         .name = NAME_STR, \
         .data = NULL, \
         .state = EN_STALE, \
         .init = en_##NAME##_init, \
         .run = en_##NAME##_run, \
-        .cleanup = en_##NAME##_cleanup, \
-        .is_valid = IS_VALID, \
-        .clear_tracked_data = CLEAR_TRACKED_DATA, \
-    };
+        .cleanup = en_##NAME##_cleanup,
 
-#define ENGINE_NODE(NAME, NAME_STR) \
-    ENGINE_NODE_DEF(NAME, NAME_STR, NULL, NULL)
+#define ENGINE_NODE_DEF_END };
 
-#define ENGINE_NODE_WITH_CLEAR_TRACK_DATA(NAME, NAME_STR) \
-    ENGINE_NODE_DEF(NAME, NAME_STR, en_##NAME##_clear_tracked_data, NULL)
+#define ENGINE_NODE1(NAME) \
+    ENGINE_NODE_DEF_START(NAME, #NAME) \
+    ENGINE_NODE_DEF_END
 
-#define ENGINE_NODE_WITH_CLEAR_TRACK_DATA_IS_VALID(NAME, NAME_STR) \
-    ENGINE_NODE_DEF(NAME, NAME_STR, \
-                    en_##NAME##_clear_tracked_data, \
-                    en_##NAME##_is_valid)
+#define CLEAR_TRACKED_DATA(NAME) \
+    .clear_tracked_data = en_##NAME##_clear_tracked_data
+
+#define IS_VALID(NAME) \
+    .is_valid = en_##NAME##_is_valid
+
+#define COMPUTE_FAIL_INFO(NAME) \
+        .get_compute_failure_info = en_##NAME##_compute_failure_info,
+
+#define ENGINE_NODE2(NAME, ARG1) \
+    ENGINE_NODE_DEF_START(NAME, #NAME) \
+    ARG1(NAME), \
+    ENGINE_NODE_DEF_END
+
+#define ENGINE_NODE3(NAME, ARG1, ARG2) \
+    ENGINE_NODE_DEF_START(NAME, #NAME) \
+    ARG1(NAME), \
+    ARG2(NAME), \
+    ENGINE_NODE_DEF_END
+
+#define ENGINE_NODE(...) VFUNC(ENGINE_NODE, __VA_ARGS__)
 
 /* Macro to define member functions of an engine node which represents
  * a table of OVSDB */
 #define ENGINE_FUNC_OVSDB(DB_NAME, TBL_NAME) \
-static void \
+static enum engine_node_state \
 en_##DB_NAME##_##TBL_NAME##_run(struct engine_node *node, \
                                 void *data OVS_UNUSED) \
 { \
     const struct DB_NAME##rec_##TBL_NAME##_table *table = \
         EN_OVSDB_GET(node); \
     if (DB_NAME##rec_##TBL_NAME##_table_track_get_first(table)) { \
-        engine_set_node_state(node, EN_UPDATED); \
-        return; \
+        return EN_UPDATED; \
     } \
-    engine_set_node_state(node, EN_UNCHANGED); \
+    return EN_UNCHANGED; \
 } \
 static void *en_##DB_NAME##_##TBL_NAME##_init( \
     struct engine_node *node OVS_UNUSED, \
@@ -447,6 +473,42 @@ static void *en_##DB_NAME##_##TBL_NAME##_init( \
 } \
 static void en_##DB_NAME##_##TBL_NAME##_cleanup(void *data OVS_UNUSED) \
 { \
+} \
+static void \
+en_##DB_NAME##_##TBL_NAME##_compute_failure_info(struct engine_node *node)  \
+{                                                                           \
+    if (!VLOG_IS_DBG_ENABLED()) {                                           \
+        return;                                                             \
+    }                                                                       \
+    const struct ovsdb_idl *table = EN_OVSDB_GET(node);                     \
+    const struct ovsdb_idl_row *row;                                        \
+    struct ds s = DS_EMPTY_INITIALIZER;                                     \
+    ds_put_format(&s, "Node \"%s\" compute failure info:\n", node->name);   \
+    for ((row) = ovsdb_idl_track_get_first(table,                           \
+                                           &DB_NAME##rec_table_##TBL_NAME); \
+         (row); (row) = ovsdb_idl_track_get_next(row)) {                    \
+        if (ovsdb_idl_row_get_seqno((row), OVSDB_IDL_CHANGE_INSERT) > 0) {  \
+            ds_put_format(&s, "%s (New) "UUID_FMT"\n",                      \
+                          #DB_NAME"_"#TBL_NAME, UUID_ARGS(&row->uuid));     \
+        } else if (ovsdb_idl_row_get_seqno((row),                           \
+                                           OVSDB_IDL_CHANGE_DELETE) > 0) {  \
+            ds_put_format(&s, "%s (Deleted) "UUID_FMT"\n",                  \
+                          #DB_NAME"_"#TBL_NAME, UUID_ARGS(&row->uuid));     \
+        } else {                                                            \
+            ds_put_format(&s, "%s (Updated) "UUID_FMT" columns:",           \
+                          #DB_NAME"_"#TBL_NAME, UUID_ARGS(&row->uuid));     \
+            for (size_t i = 0; i < row->table->class_->n_columns; i++) {    \
+                struct ovsdb_idl_column *col =                              \
+                    &DB_NAME##rec_##TBL_NAME##_columns[i];                  \
+                if (ovsdb_idl_track_is_updated(row, col)) {                 \
+                    ds_put_format(&s, " %s,", col->name);                   \
+                }                                                           \
+            }                                                               \
+            ds_chomp(&s, ',');                                              \
+        }                                                                   \
+    }                                                                       \
+    VLOG_DBG("%s", ds_cstr(&s));                                            \
+    ds_destroy(&s);                                                         \
 }
 
 /* Macro to define member functions of an engine node which represents
@@ -466,19 +528,21 @@ static void en_##DB_NAME##_##TBL_NAME##_cleanup(void *data OVS_UNUSED) \
 
 /* Macro to define an engine node which represents a table of OVSDB */
 #define ENGINE_NODE_OVSDB(DB_NAME, DB_NAME_STR, TBL_NAME, TBL_NAME_STR) \
-    ENGINE_NODE(DB_NAME##_##TBL_NAME, DB_NAME_STR"_"TBL_NAME_STR)
+    ENGINE_NODE_DEF_START(DB_NAME##_##TBL_NAME, DB_NAME_STR"_"TBL_NAME_STR) \
+    COMPUTE_FAIL_INFO(DB_NAME##_##TBL_NAME) \
+    ENGINE_NODE_DEF_END
 
 /* Macro to define an engine node which represents a table of OVN SB DB */
-#define ENGINE_NODE_SB(TBL_NAME, TBL_NAME_STR) \
-    ENGINE_NODE_OVSDB(sb, "SB", TBL_NAME, TBL_NAME_STR);
+#define ENGINE_NODE_SB(TBL_NAME) \
+    ENGINE_NODE_OVSDB(sb, "SB", TBL_NAME, #TBL_NAME);
 
 /* Macro to define an engine node which represents a table of OVN NB DB */
-#define ENGINE_NODE_NB(TBL_NAME, TBL_NAME_STR) \
-    ENGINE_NODE_OVSDB(nb, "NB", TBL_NAME, TBL_NAME_STR);
+#define ENGINE_NODE_NB(TBL_NAME) \
+    ENGINE_NODE_OVSDB(nb, "NB", TBL_NAME, #TBL_NAME);
 
 /* Macro to define an engine node which represents a table of open_vswitch
  * DB */
-#define ENGINE_NODE_OVS(TBL_NAME, TBL_NAME_STR) \
-    ENGINE_NODE_OVSDB(ovs, "OVS", TBL_NAME, TBL_NAME_STR);
+#define ENGINE_NODE_OVS(TBL_NAME) \
+    ENGINE_NODE_OVSDB(ovs, "OVS", TBL_NAME, #TBL_NAME);
 
 #endif /* lib/inc-proc-eng.h */
