@@ -27,4 +27,10 @@ const struct sbrec_mac_binding *
 mac_binding_lookup(struct ovsdb_idl_index *sbrec_mac_binding_by_lport_ip,
                    const char *logical_port, const char *ip);
 
+void mac_binding_add_to_sb(
+    struct ovsdb_idl_txn *,
+    struct ovsdb_idl_index *sbrec_mac_binding_by_lport_ip,
+    const char *logical_port, const struct sbrec_datapath_binding *,
+    struct eth_addr, const char *ip, bool update_only);
+
 #endif /* lib/mac-binding-index.h */
