@@ -16,6 +16,7 @@
 #ifndef OVN_UTIL_H
 #define OVN_UTIL_H 1
 
+#include "openvswitch/meta-flow.h"
 #include "ovsdb-idl.h"
 #include "lib/packets.h"
 #include "lib/sset.h"
@@ -30,12 +31,12 @@
 
 #define ROUTE_ORIGIN_CONNECTED "connected"
 #define ROUTE_ORIGIN_STATIC "static"
+#define ROUTE_ORIGIN_LB "loadbalancer"
 
 #define ETH_CRC_LENGTH 4
 #define ETHERNET_OVERHEAD (ETH_HEADER_LEN + ETH_CRC_LENGTH)
 
 #define GENEVE_TUNNEL_OVERHEAD 38
-#define STT_TUNNEL_OVERHEAD 18
 #define VXLAN_TUNNEL_OVERHEAD 30
 
 struct eth_addr;
@@ -167,6 +168,9 @@ void set_idl_probe_interval(struct ovsdb_idl *idl, const char *remote,
 #define OVN_MIN_DP_VXLAN_KEY_GLOBAL (OVN_MAX_DP_VXLAN_KEY_LOCAL + 1)
 #define OVN_MAX_DP_VXLAN_KEY_GLOBAL ((1u << 12) - 1)
 
+#define OVN_MIN_EVPN_KEY (1u << 31)
+#define OVN_MAX_EVPN_KEY (OVN_MAX_DP_GLOBAL_NUM | OVN_MIN_EVPN_KEY)
+
 struct hmap;
 void ovn_destroy_tnlids(struct hmap *tnlids);
 bool ovn_add_tnlid(struct hmap *set, uint32_t tnlid);
@@ -200,6 +204,8 @@ get_sb_port_group_name(const char *nb_pg_name, int64_t dp_tunnel_key,
 }
 
 char *ovn_chassis_redirect_name(const char *port_name);
+char *ovn_mirror_port_name(const char *datapath_name,
+                           const char *port_name);
 void ovn_set_pidfile(const char *name);
 
 bool ip46_parse_cidr(const char *str, struct in6_addr *prefix,
@@ -218,6 +224,7 @@ char *str_tolower(const char *orig);
 
 long long ovn_smap_get_llong(const struct smap *smap, const char *key,
                              long long def);
+bool ovn_str_to_ushort(const char *, int base, unsigned short *);
 
 /* OVN daemon options. Taken from ovs/lib/daemon.h. */
 #define OVN_DAEMON_OPTION_ENUMS                     \
@@ -313,7 +320,8 @@ BUILD_ASSERT_DECL(
 #define SCTP_ABORT_CHUNK_FLAG_T (1 << 0)
 
 /* The number of tables for the ingress and egress pipelines. */
-#define LOG_PIPELINE_LEN 30
+#define LOG_PIPELINE_INGRESS_LEN 32
+#define LOG_PIPELINE_EGRESS_LEN 14
 
 static inline uint32_t
 hash_add_in6_addr(uint32_t hash, const struct in6_addr *addr)
@@ -336,9 +344,7 @@ hash_add_in6_addr(uint32_t hash, const struct in6_addr *addr)
 /* Must be a bit-field ordered from most-preferred (higher number) to
  * least-preferred (lower number). */
 enum chassis_tunnel_type {
-    GENEVE = 1 << 2,
-    STT    = 1 << 1,  /* NOTE: STT support is deprecated and will be removed
-                       *       in an upcoming release. */
+    GENEVE = 1 << 1,
     VXLAN  = 1 << 0
 };
 
@@ -409,6 +415,7 @@ enum en_lport_type {
     LP_UNKNOWN,
     LP_VIF,
     LP_CONTAINER,
+    LP_MIRROR,
     LP_PATCH,
     LP_L3GATEWAY,
     LP_LOCALNET,
@@ -492,10 +499,65 @@ bool ovn_update_swconn_at(struct rconn *swconn, const char *target,
 
 bool prefix_is_link_local(const struct in6_addr *prefix, unsigned int plen);
 
+bool find_prefix_in_set(const struct in6_addr *prefix, unsigned int plen,
+                        const struct sset *prefix_set,
+                        const char *filter_name);
+
 void ovn_debug_commands_register(void);
 
+bool ovn_is_valid_vni(int64_t vni);
+
 const struct sbrec_port_binding *lport_lookup_by_name(
     struct ovsdb_idl_index *sbrec_port_binding_by_name,
     const char *name);
 
+void put_load_bytes(const void *value, size_t len, enum mf_field_id dst,
+                    size_t ofs, size_t n_bits, struct ofpbuf *ofpacts);
+void put_load(uint64_t value, enum mf_field_id dst, size_t ofs, size_t n_bits,
+              struct ofpbuf *ofpacts);
+
+/* __NARG__ provides a way to count the number of arguments passed to a
+ * variadic macro. As defined below, it's capable of counting up to
+ * 16 arguments. This should be more than enough for our purposes. However
+ * the __ARG_N and __RSEQ_N macros can be updated to include more numbers
+ * if we wish to be able to count higher.
+ * */
+#define __NARG__(...)  __NARG_I_(__VA_ARGS__,__RSEQ_N())
+#define __NARG_I_(...) __ARG_N(__VA_ARGS__)
+#define __ARG_N( \
+      _1, _2, _3, _4, _5, _6, _7, _8, _9, \
+      _10, _11, _12, _13, _14, _15, _16, N,...) N
+#define __RSEQ_N() \
+     16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0
+
+/* VFUNC provides a way to overload macros so that they can
+ * accept a variable number of arguments.
+ *
+ * It works by using __NARG__ to count the number of arguments
+ * that were passed into the macro. Then this is concatenated
+ * to the end of the macro name. Then this concatenated macro
+ * is called.
+ *
+ * As an example, let's say we have
+ * # define FOO(...) VFUNC(FOO, __VA_ARGS)
+ *
+ * Then if a caller were to call FOO(bar)
+ * then we would count one argument (bar), concatenate
+ * the 1 to the end of FOO, and end up calling FOO1(bar).
+ *
+ * If a caller were to call FOO(bar, baz), then the
+ * result would be to call FOO2(bar, baz).
+ */
+#define _VFUNC_(name, n) name##n
+#define _VFUNC(name, n) _VFUNC_(name, n)
+#define VFUNC(func, ...) _VFUNC(func, __NARG__(__VA_ARGS__)) (__VA_ARGS__)
+
+bool datapath_get_nb_uuid_and_type(const struct sbrec_datapath_binding *sb,
+                                   struct uuid *nb_uuid, const char **type);
+
+bool datapath_get_nb_uuid(const struct sbrec_datapath_binding *sb,
+                          struct uuid *nb_uuid);
+
+const char *datapath_get_nb_type(const struct sbrec_datapath_binding *sb);
+
 #endif /* OVN_UTIL_H */
