@@ -19,13 +19,15 @@
 #include "lib/inc-proc-eng.h"
 
 /* The MAC binding aging node functions. */
-void en_mac_binding_aging_run(struct engine_node *node, void *data);
+enum engine_node_state en_mac_binding_aging_run(struct engine_node *node,
+                                                void *data);
 void *en_mac_binding_aging_init(struct engine_node *node,
                                 struct engine_arg *arg);
 void en_mac_binding_aging_cleanup(void *data);
 
 /* The MAC binding aging waker node functions. */
-void en_mac_binding_aging_waker_run(struct engine_node *node, void *data);
+enum engine_node_state en_mac_binding_aging_waker_run(struct engine_node *node,
+                                                      void *data);
 void *en_mac_binding_aging_waker_init(struct engine_node *node,
                                       struct engine_arg *arg);
 void en_mac_binding_aging_waker_cleanup(void *data);
@@ -34,12 +36,13 @@ void en_mac_binding_aging_waker_cleanup(void *data);
 unsigned int min_mac_binding_age_threshold(const char *opt);
 
 /* The FDB aging node functions. */
-void en_fdb_aging_run(struct engine_node *node, void *data);
+enum engine_node_state en_fdb_aging_run(struct engine_node *node, void *data);
 void *en_fdb_aging_init(struct engine_node *node, struct engine_arg *arg);
 void en_fdb_aging_cleanup(void *data);
 
 /* The FDB aging waker node functions. */
-void en_fdb_aging_waker_run(struct engine_node *node, void *data);
+enum engine_node_state en_fdb_aging_waker_run(struct engine_node *node,
+                                              void *data);
 void *en_fdb_aging_waker_init(struct engine_node *node,
                               struct engine_arg *arg);
 void en_fdb_aging_waker_cleanup(void *data);
