@@ -56,7 +56,7 @@ ar_entry_add_nocopy(struct hmap *routes, const struct ovn_datapath *od,
     route_e->ip_prefix = ip_prefix;
     route_e->tracked_port = tracked_port;
     route_e->source = source;
-    uint32_t hash = uuid_hash(&od->sb->header_.uuid);
+    uint32_t hash = uuid_hash(&od->sdp->sb_dp->header_.uuid);
     hash = hash_string(op->sb->logical_port, hash);
     hash = hash_string(ip_prefix, hash);
     hmap_insert(routes, &route_e->hmap_node, hash);
@@ -92,7 +92,7 @@ ar_entry_find(struct hmap *route_map,
 
     HMAP_FOR_EACH_WITH_HASH (route_e, hmap_node, hash, route_map) {
         if (!uuid_equals(&sb_db->header_.uuid,
-                         &route_e->od->sb->header_.uuid)) {
+                         &route_e->od->sdp->sb_dp->header_.uuid)) {
             continue;
         }
         if (!uuid_equals(&logical_port->header_.uuid,
@@ -127,12 +127,11 @@ static void
 advertised_route_table_sync(
     struct ovsdb_idl_txn *ovnsb_txn,
     const struct sbrec_advertised_route_table *sbrec_advertised_route_table,
-    const struct lr_stateful_table *lr_stateful_table,
     const struct hmap *routes,
     const struct hmap *dynamic_routes,
     struct advertised_route_sync_data *data);
 
-bool
+enum engine_input_handler_result
 advertised_route_sync_lr_stateful_change_handler(struct engine_node *node,
                                                  void *data_)
 {
@@ -149,21 +148,21 @@ advertised_route_sync_lr_stateful_change_handler(struct engine_node *node,
         lr_stateful_rec = hmapx_node->data;
         if (uuidset_contains(&data->nb_lr,
                              &lr_stateful_rec->nbr_uuid)) {
-            return false;
+            return EN_UNHANDLED;
         }
     }
 
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
-bool
+enum engine_input_handler_result
 advertised_route_sync_northd_change_handler(struct engine_node *node,
                                             void *data_)
 {
     struct advertised_route_sync_data *data = data_;
     struct northd_data *northd_data = engine_get_input_data("northd", node);
     if (!northd_has_tracked_data(&northd_data->trk_data)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     /* We indirectly use northd_data->ls_ports if we announce host routes.
@@ -175,25 +174,25 @@ advertised_route_sync_northd_change_handler(struct engine_node *node,
         op = hmapx_node->data;
         if (uuidset_contains(&data->nb_ls,
                              &op->od->nbs->header_.uuid)) {
-            return false;
+            return EN_UNHANDLED;
         }
     }
     HMAPX_FOR_EACH (hmapx_node, &northd_data->trk_data.trk_lsps.updated) {
         op = hmapx_node->data;
         if (uuidset_contains(&data->nb_ls,
                              &op->od->nbs->header_.uuid)) {
-            return false;
+            return EN_UNHANDLED;
         }
     }
     HMAPX_FOR_EACH (hmapx_node, &northd_data->trk_data.trk_lsps.deleted) {
         op = hmapx_node->data;
         if (uuidset_contains(&data->nb_ls,
                              &op->od->nbs->header_.uuid)) {
-            return false;
+            return EN_UNHANDLED;
         }
     }
 
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
 static void
@@ -232,7 +231,7 @@ en_advertised_route_sync_cleanup(void *data OVS_UNUSED)
     routes_sync_destroy(data);
 }
 
-void
+enum engine_node_state
 en_advertised_route_sync_run(struct engine_node *node, void *data OVS_UNUSED)
 {
     routes_sync_clear(data);
@@ -245,20 +244,17 @@ en_advertised_route_sync_run(struct engine_node *node, void *data OVS_UNUSED)
     const struct engine_context *eng_ctx = engine_get_context();
     const struct sbrec_advertised_route_table *sbrec_advertised_route_table =
         EN_OVSDB_GET(engine_get_input("SB_advertised_route", node));
-    struct ed_type_lr_stateful *lr_stateful_data =
-        engine_get_input_data("lr_stateful", node);
 
     stopwatch_start(ADVERTISED_ROUTE_SYNC_RUN_STOPWATCH_NAME, time_msec());
 
     advertised_route_table_sync(eng_ctx->ovnsb_idl_txn,
                                 sbrec_advertised_route_table,
-                                &lr_stateful_data->table,
                                 &routes_data->parsed_routes,
                                 &dynamic_routes_data->routes,
                                 routes_sync_data);
 
     stopwatch_stop(ADVERTISED_ROUTE_SYNC_RUN_STOPWATCH_NAME, time_msec());
-    engine_set_node_state(node, EN_UPDATED);
+    return EN_UPDATED;
 }
 
 /* This function adds a new route for each entry in lr_nat record
@@ -285,7 +281,8 @@ build_nat_route_for_port(const struct ovn_port *advertising_op,
             ? ovn_port_find(ls_ports, nat->nb->logical_port)
             : nat->l3dgw_port;
 
-        if (!ar_entry_find(routes, advertising_od->sb, advertising_op->sb,
+        if (!ar_entry_find(routes, advertising_od->sdp->sb_dp,
+                           advertising_op->sb,
                            nat->nb->external_ip,
                            tracked_port ? tracked_port->sb : NULL)) {
             ar_entry_add(routes, advertising_od, advertising_op,
@@ -338,11 +335,8 @@ build_nat_connected_routes(
         struct ovn_datapath *peer_od = op->peer->od;
         ovs_assert(peer_od->nbs || peer_od->nbr);
 
-        const struct ovn_port *peer_port = NULL;
         /* This is a directly connected LR peer. */
         if (peer_od->nbr) {
-            peer_port = op->peer;
-
             const struct lr_stateful_record *peer_lr_stateful =
                 lr_stateful_table_find_by_index(lr_stateful_table,
                                                  peer_od->index);
@@ -353,21 +347,21 @@ build_nat_connected_routes(
             /* Advertise peer's NAT routes via the local port too. */
             build_nat_route_for_port(op, peer_lr_stateful->lrnat_rec,
                                      ls_ports, routes);
-            return;
+            continue;
         }
 
         /* This peer is LSP, we need to check all connected router ports
          * for NAT.*/
-        for (size_t i = 0; i < peer_od->n_router_ports; i++) {
-            peer_port = peer_od->router_ports[i]->peer;
-            if (peer_port == op) {
+        const struct ovn_port *rp;
+        VECTOR_FOR_EACH (&peer_od->router_ports, rp) {
+            if (rp->peer == op) {
                 /* Skip advertising router. */
                 continue;
             }
 
             const struct lr_stateful_record *peer_lr_stateful =
                 lr_stateful_table_find_by_index(lr_stateful_table,
-                                                peer_port->od->index);
+                                                rp->peer->od->index);
             if (!peer_lr_stateful) {
                 continue;
             }
@@ -423,30 +417,28 @@ build_lb_connected_routes(const struct ovn_datapath *od,
         ovs_assert(peer_od->nbs || peer_od->nbr);
 
         const struct lr_stateful_record *lr_stateful_rec;
-        const struct ovn_port *peer_port = NULL;
         /* This is directly connected LR peer. */
         if (peer_od->nbr) {
             lr_stateful_rec = lr_stateful_table_find_by_index(
                 lr_stateful_table, peer_od->index);
-            peer_port = op->peer;
-            build_lb_route_for_port(op, peer_port, lr_stateful_rec->lb_ips,
+            build_lb_route_for_port(op, op->peer, lr_stateful_rec->lb_ips,
                                     routes);
-            return;
+            continue;
         }
 
         /* This peer is LSP, we need to check all connected router ports for
          * LBs.*/
-        for (size_t i = 0; i < peer_od->n_router_ports; i++) {
-            peer_port = peer_od->router_ports[i]->peer;
-            if (peer_port == op) {
+        struct ovn_port *rp;
+        VECTOR_FOR_EACH (&peer_od->router_ports, rp) {
+            if (rp->peer == op) {
                 /* no need to check for LBs on ovn_port that initiated this
                  * function.*/
                 continue;
             }
             lr_stateful_rec = lr_stateful_table_find_by_index(
-                lr_stateful_table, peer_port->od->index);
+                lr_stateful_table, rp->peer->od->index);
 
-            build_lb_route_for_port(op, peer_port, lr_stateful_rec->lb_ips,
+            build_lb_route_for_port(op, rp->peer, lr_stateful_rec->lb_ips,
                                     routes);
         }
     }
@@ -470,14 +462,14 @@ build_lb_routes(const struct ovn_datapath *od,
          *
          * Advertise the LB IPs via all 'op' if this is a gateway router or
          * throuh all DGPs of this distributed router otherwise. */
-        struct ovn_port *op_ = NULL;
-        size_t n_tracked_ports = !od->is_gw_router ? od->n_l3dgw_ports : 1;
-        struct ovn_port **tracked_ports = !od->is_gw_router
-                                          ? od->l3dgw_ports
-                                          : &op_;
 
-        for (size_t i = 0; i < n_tracked_ports; i++) {
-            build_lb_route_for_port(op, tracked_ports[i], lb_ips, routes);
+        if (od->is_gw_router) {
+            build_lb_route_for_port(op, NULL, lb_ips, routes);
+        } else {
+            struct ovn_port *dgp;
+            VECTOR_FOR_EACH (&od->l3dgw_ports, dgp) {
+                build_lb_route_for_port(op, dgp, lb_ips, routes);
+            }
         }
     }
 }
@@ -511,7 +503,7 @@ en_dynamic_routes_cleanup(void *data_)
     hmap_destroy(&data->routes);
 }
 
-void
+enum engine_node_state
 en_dynamic_routes_run(struct engine_node *node, void *data)
 {
     struct dynamic_routes_data *dynamic_routes_data = data;
@@ -544,7 +536,7 @@ en_dynamic_routes_run(struct engine_node *node, void *data)
                                   &dynamic_routes_data->routes);
     }
     stopwatch_stop(DYNAMIC_ROUTES_RUN_STOPWATCH_NAME, time_msec());
-    engine_set_node_state(node, EN_UPDATED);
+    return EN_UPDATED;
 }
 
 static void
@@ -569,40 +561,10 @@ publish_lport_addresses(struct hmap *sync_routes,
     }
 }
 
-/* Collect all IP addresses connected via this LRP. */
-static void
-publish_host_routes_lrp(struct hmap *sync_routes,
-                        const struct lr_stateful_table *lr_stateful_table,
-                        const struct ovn_datapath *advertising_od,
-                        const struct ovn_port *advertising_op,
-                        struct advertised_route_sync_data *data,
-                        const struct ovn_port *lrp)
-{
-    /* This is a LSP connected to an LRP */
-    const struct lport_addresses *addresses = &lrp->lrp_networks;
-    publish_lport_addresses(sync_routes, advertising_od,
-                            advertising_op, addresses, lrp);
-
-    const struct lr_stateful_record *lr_stateful_rec;
-    lr_stateful_rec =
-        lr_stateful_table_find_by_index(lr_stateful_table, lrp->od->index);
-    /* We also need to track this LR as we need to recompute when
-     * any of its IPs change. */
-    uuidset_insert(&data->nb_lr, &lr_stateful_rec->nbr_uuid);
-    struct ovn_port_routable_addresses addrs =
-        get_op_addresses(lrp, lr_stateful_rec, false);
-    for (size_t i = 0; i < addrs.n_addrs; i++) {
-        publish_lport_addresses(sync_routes, advertising_od,
-                                advertising_op, &addrs.laddrs[i], lrp);
-    }
-    destroy_routable_addresses(&addrs);
-}
-
 /* Collect all IP addresses connected to the out_port of a route.
  * This traverses all LSPs on the LS connected to the out_port. */
 static void
 publish_host_routes(struct hmap *sync_routes,
-                    const struct lr_stateful_table *lr_stateful_table,
                     const struct ovn_datapath *advertising_od,
                     const struct ovn_port *advertising_op,
                     struct advertised_route_sync_data *data)
@@ -618,8 +580,9 @@ publish_host_routes(struct hmap *sync_routes,
 
     if (peer_od->nbr) {
         /* This is a LRP directly connected to another LRP. */
-        publish_host_routes_lrp(sync_routes, lr_stateful_table, advertising_od,
-                                advertising_op, data, advertising_op->peer);
+        const struct ovn_port *lrp = advertising_op->peer;
+        publish_lport_addresses(sync_routes, advertising_od,
+                                advertising_op, &lrp->lrp_networks, lrp);
         return;
     }
 
@@ -631,9 +594,9 @@ publish_host_routes(struct hmap *sync_routes,
     HMAP_FOR_EACH (port, dp_node, &peer_od->ports) {
         if (port->peer && port->peer->nbrp) {
             /* This is a LSP connected to an LRP */
-            publish_host_routes_lrp(sync_routes, lr_stateful_table,
-                                    advertising_od, advertising_op,
-                                    data, port->peer);
+            const struct ovn_port *lrp = port->peer;
+            publish_lport_addresses(sync_routes, advertising_od,
+                                    advertising_op, &lrp->lrp_networks, lrp);
         } else {
             /* This is just a plain LSP */
             for (size_t i = 0; i < port->n_lsp_addrs; i++) {
@@ -686,38 +649,44 @@ should_advertise_route(const struct uuidset *host_route_lrps,
     }
 }
 
-/* Returns true if the route has already been fully processed for
- * advertising (e.g., for connected routes when connected-as-host is set).
- * Returns false otherwise, in which case the a new 'ar_entry' needs to
- * be created for this route.
- */
+/* Returns true if the connected route was advertised as a set of host routes
+ * (/32 for IPv4 and /128 for IPv6), one for each individual IP known to be
+ * reachable in the connected route's subnet.  Returns false otherwise. */
 static bool
-process_prereqs_advertise_route(
-    const struct lr_stateful_table *lr_stateful_table,
+advertise_routes_as_host_prefix(
     struct advertised_route_sync_data *data,
     struct uuidset *host_route_lrps,
     struct hmap *sync_routes,
     const struct ovn_datapath *advertising_od,
     const struct ovn_port *advertising_op,
-    const struct ovn_port *tracked_op,
-    enum route_source source)
+    enum route_source source
+)
 {
+    if (source != ROUTE_SOURCE_CONNECTED) {
+        return false;
+    }
+
     enum dynamic_routing_redistribute_mode drr =
         advertising_op->dynamic_routing_redistribute;
+    if (!drr_mode_CONNECTED_AS_HOST_is_set(drr)) {
+        return false;
+    }
 
+    uuidset_insert(host_route_lrps, &advertising_op->nbrp->header_.uuid);
+    publish_host_routes(sync_routes, advertising_od, advertising_op, data);
+    return true;
+}
+
+/* Track datapaths (routers/switches) whose changes should trigger
+ * the set of advertised routes.  That includes NAT and LB related
+ * advertised routes. */
+static void
+advertise_route_track_od(struct advertised_route_sync_data *data,
+                         const struct ovn_datapath *advertising_od,
+                         const struct ovn_port *tracked_op,
+                         enum route_source source)
+{
     switch (source) {
-    case ROUTE_SOURCE_CONNECTED:
-        if (drr_mode_CONNECTED_AS_HOST_is_set(drr)) {
-            const struct uuid *lrp_uuid = &advertising_op->nbrp->header_.uuid;
-            uuidset_insert(host_route_lrps, lrp_uuid);
-            publish_host_routes(sync_routes, lr_stateful_table,
-                                advertising_od, advertising_op,
-                                data);
-            return true;
-        }
-        break;
-    case ROUTE_SOURCE_STATIC:
-        break;
     case ROUTE_SOURCE_NAT:
         /* If NAT route tracks port on a different DP than the one that
          * advertises the route, we need to watch for changes on that DP as
@@ -741,19 +710,20 @@ process_prereqs_advertise_route(
                            &tracked_op->od->nbr->header_.uuid);
         }
         break;
+    case ROUTE_SOURCE_CONNECTED:
+    case ROUTE_SOURCE_STATIC:
+        break;
     case ROUTE_SOURCE_LEARNED:
         OVS_NOT_REACHED();
     default:
         OVS_NOT_REACHED();
     }
-    return false;
 }
 
 static void
 advertised_route_table_sync(
     struct ovsdb_idl_txn *ovnsb_txn,
     const struct sbrec_advertised_route_table *sbrec_advertised_route_table,
-    const struct lr_stateful_table *lr_stateful_table,
     const struct hmap *routes,
     const struct hmap *dynamic_routes,
     struct advertised_route_sync_data *data)
@@ -768,23 +738,25 @@ advertised_route_table_sync(
             continue;
         }
 
-        if (prefix_is_link_local(&route->prefix, route->plen)) {
-            continue;
-        }
-
         if (!should_advertise_route(&host_route_lrps, route->od,
                                     route->out_port, route->source)) {
             continue;
         }
 
-        if (process_prereqs_advertise_route(lr_stateful_table, data,
-                                            &host_route_lrps, &sync_routes,
-                                            route->od, route->out_port,
-                                            route->tracked_port,
+        if (advertise_routes_as_host_prefix(data, &host_route_lrps,
+                                            &sync_routes, route->od,
+                                            route->out_port,
                                             route->source)) {
             continue;
         }
 
+        if (prefix_is_link_local(&route->prefix, route->plen)) {
+            continue;
+        }
+
+        advertise_route_track_od(data, route->od, route->tracked_port,
+                                 route->source);
+
         ar_entry_add_nocopy(&sync_routes, route->od, route->out_port,
                             normalize_v46_prefix(&route->prefix, route->plen),
                             route->tracked_port,
@@ -799,14 +771,19 @@ advertised_route_table_sync(
             continue;
         }
 
-        if (process_prereqs_advertise_route(lr_stateful_table, data,
-                                            &host_route_lrps, &sync_routes,
-                                            route_e->od, route_e->op,
-                                            route_e->tracked_port,
-                                            route_e->source)) {
+        advertise_route_track_od(data, route_e->od, route_e->tracked_port,
+                                 route_e->source);
+
+        const struct sbrec_port_binding *tracked_pb =
+            route_e->tracked_port ? route_e->tracked_port->sb : NULL;
+        if (ar_entry_find(&sync_routes, route_e->od->sdp->sb_dp,
+                          route_e->op->sb,
+                          route_e->ip_prefix, tracked_pb)) {
+            /* We could already have advertised route entry for LRP IP that
+             * corresponds to "snat" when "connected-as-host" is combined
+             * with "nat". Skip it. */
             continue;
         }
-
         ar_entry_add(&sync_routes, route_e->od, route_e->op,
                      route_e->ip_prefix, route_e->tracked_port,
                      route_e->source);
@@ -819,18 +796,23 @@ advertised_route_table_sync(
         route_e = ar_entry_find(&sync_routes, sb_route->datapath,
                                 sb_route->logical_port, sb_route->ip_prefix,
                                 sb_route->tracked_port);
-        if (route_e) {
-          hmap_remove(&sync_routes, &route_e->hmap_node);
-          ar_entry_free(route_e);
-        } else {
-          sbrec_advertised_route_delete(sb_route);
+        if (!route_e) {
+            sbrec_advertised_route_delete(sb_route);
+            continue;
         }
+
+        if (route_e->tracked_port && !sb_route->tracked_port) {
+            sbrec_advertised_route_set_tracked_port(
+                sb_route, route_e->tracked_port->sb);
+        }
+        hmap_remove(&sync_routes, &route_e->hmap_node);
+        ar_entry_free(route_e);
     }
 
     HMAP_FOR_EACH_POP (route_e, hmap_node, &sync_routes) {
         const struct sbrec_advertised_route *sr =
             sbrec_advertised_route_insert(ovnsb_txn);
-        sbrec_advertised_route_set_datapath(sr, route_e->od->sb);
+        sbrec_advertised_route_set_datapath(sr, route_e->od->sdp->sb_dp);
         sbrec_advertised_route_set_logical_port(sr, route_e->op->sb);
         sbrec_advertised_route_set_ip_prefix(sr, route_e->ip_prefix);
         if (route_e->tracked_port) {
