@@ -49,6 +49,8 @@ static bool check_nb_options_out_of_sync(
     const struct sampling_app_table *);
 static void update_sb_config_options_to_sbrec(struct ed_type_global_config *,
                                               const struct sbrec_sb_global *);
+static bool is_vxlan_mode(const struct smap *nb_options,
+                          const struct sbrec_chassis_table *);
 
 void *
 en_global_config_init(struct engine_node *node OVS_UNUSED,
@@ -61,14 +63,21 @@ en_global_config_init(struct engine_node *node OVS_UNUSED,
     return data;
 }
 
-void
+static uint32_t
+get_ovn_max_dp_key_local(bool vxlan_mode, bool vxlan_ic_mode)
+{
+    if (vxlan_mode) {
+        /* OVN_MAX_DP_GLOBAL_NUM doesn't apply for VXLAN mode. */
+        return vxlan_ic_mode ? OVN_MAX_DP_VXLAN_KEY_LOCAL
+                             : OVN_MAX_DP_VXLAN_KEY;
+    }
+    return vxlan_ic_mode ? OVN_MAX_DP_VXLAN_KEY_LOCAL : OVN_MAX_DP_KEY_LOCAL;
+}
+
+enum engine_node_state
 en_global_config_run(struct engine_node *node , void *data)
 {
     const struct engine_context *eng_ctx = engine_get_context();
-    if (!eng_ctx->ovnnb_idl_txn || !eng_ctx->ovnsb_idl_txn) {
-        return;
-    }
-
     const struct nbrec_nb_global_table *nb_global_table =
         EN_OVSDB_GET(engine_get_input("NB_nb_global", node));
     const struct nbrec_logical_switch_table *nbrec_ls_table =
@@ -131,11 +140,11 @@ en_global_config_run(struct engine_node *node , void *data)
             break;
         }
     }
-    uint32_t max_dp_key =
-        get_ovn_max_dp_key_local(is_vxlan_mode(&nb->options,
-                                               sbrec_chassis_table),
-                                 ic_vxlan_mode);
-    char *max_tunid = xasprintf("%d", max_dp_key);
+    config_data->vxlan_mode = is_vxlan_mode(&nb->options, sbrec_chassis_table);
+    config_data->max_dp_tunnel_id =
+        get_ovn_max_dp_key_local(config_data->vxlan_mode, ic_vxlan_mode);
+
+    char *max_tunid = xasprintf("%d", config_data->max_dp_tunnel_id);
     smap_replace(options, "max_tunid", max_tunid);
     free(max_tunid);
 
@@ -177,7 +186,7 @@ en_global_config_run(struct engine_node *node , void *data)
     /* Set up SB_Global (depends on chassis features). */
     update_sb_config_options_to_sbrec(config_data, sb);
 
-    engine_set_node_state(node, EN_UPDATED);
+    return EN_UPDATED;
 }
 
 void en_global_config_cleanup(void *data OVS_UNUSED)
@@ -197,7 +206,7 @@ en_global_config_clear_tracked_data(void *data)
     config_data->tracked_data.chassis_features_changed = false;
 }
 
-bool
+enum engine_input_handler_result
 global_config_nb_global_handler(struct engine_node *node, void *data)
 {
     const struct nbrec_nb_global_table *nb_global_table =
@@ -211,19 +220,19 @@ global_config_nb_global_handler(struct engine_node *node, void *data)
     const struct nbrec_nb_global *nb =
         nbrec_nb_global_table_first(nb_global_table);
     if (!nb) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     const struct sbrec_sb_global *sb =
         sbrec_sb_global_table_first(sb_global_table);
     if (!sb) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     /* We are only interested in ipsec and options column. */
     if (!nbrec_nb_global_is_updated(nb, NBREC_NB_GLOBAL_COL_IPSEC)
         && !nbrec_nb_global_is_updated(nb, NBREC_NB_GLOBAL_COL_OPTIONS)) {
-        return true;
+        return EN_HANDLED_UNCHANGED;
     }
 
     if (nb->ipsec != sb->ipsec) {
@@ -234,7 +243,7 @@ global_config_nb_global_handler(struct engine_node *node, void *data)
     config_data->tracked = true;
 
     if (smap_equal(&nb->options, &config_data->nb_options)) {
-        return true;
+        return EN_HANDLED_UNCHANGED;
     }
 
     /* Return false if an option is out of sync and requires updating the
@@ -242,31 +251,36 @@ global_config_nb_global_handler(struct engine_node *node, void *data)
     /* Check if svc_monitor_mac has changed or not. */
     if (config_out_of_sync(&nb->options, &config_data->nb_options,
                            "svc_monitor_mac", true)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     /* Check if max_tunid has changed or not. */
     if (config_out_of_sync(&nb->options, &config_data->nb_options,
                            "max_tunid", true)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     /* Check if mac_prefix has changed or not. */
     if (config_out_of_sync(&nb->options, &config_data->nb_options,
                            "mac_prefix", true)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     /* Check if ignore_chassis_features has changed or not. */
     if (config_out_of_sync(&nb->options, &config_data->nb_options,
                            "ignore_chassis_features", false)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     /* Check if northd_internal_version has changed or not. */
     if (config_out_of_sync(&nb->options, &config_data->nb_options,
                            "northd_internal_version", false)) {
-        return false;
+        return EN_UNHANDLED;
+    }
+
+    if (config_out_of_sync(&nb->options, &config_data->nb_options,
+                           "vxlan_mode", false)) {
+        return EN_UNHANDLED;
     }
 
     if (check_nb_options_out_of_sync(nb, config_data, sampling_apps)) {
@@ -278,11 +292,10 @@ global_config_nb_global_handler(struct engine_node *node, void *data)
 
     update_sb_config_options_to_sbrec(config_data, sb);
 
-    engine_set_node_state(node, EN_UPDATED);
-    return true;
+    return EN_HANDLED_UPDATED;
 }
 
-bool
+enum engine_input_handler_result
 global_config_sb_global_handler(struct engine_node *node, void *data)
 {
     const struct sbrec_sb_global_table *sb_global_table =
@@ -291,20 +304,20 @@ global_config_sb_global_handler(struct engine_node *node, void *data)
     const struct sbrec_sb_global *sb =
         sbrec_sb_global_table_first(sb_global_table);
     if (!sb) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     struct ed_type_global_config *config_data = data;
 
     if (!smap_equal(&sb->options, &config_data->sb_options)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     /* No need to update the engine node. */
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
-bool
+enum engine_input_handler_result
 global_config_sb_chassis_handler(struct engine_node *node, void *data)
 {
     struct ed_type_global_config *config_data = data;
@@ -318,20 +331,20 @@ global_config_sb_chassis_handler(struct engine_node *node, void *data)
             || sbrec_chassis_is_deleted(chassis)
             || sbrec_chassis_is_updated(chassis,
                                         SBREC_CHASSIS_COL_ENCAPS)) {
-            return false;
+            return EN_UNHANDLED;
         }
 
         for (size_t i = 0; i < chassis->n_encaps; i++) {
             if (sbrec_encap_row_get_seqno(chassis->encaps[i],
                                           OVSDB_IDL_CHANGE_MODIFY) > 0) {
-                return false;
+                return EN_UNHANDLED;
             }
         }
     }
 
     if (smap_get_bool(&config_data->nb_options, "ignore_chassis_features",
                       false)) {
-        return true;
+        return EN_HANDLED_UNCHANGED;
     }
 
     bool reevaluate_chassis_features = false;
@@ -346,7 +359,7 @@ global_config_sb_chassis_handler(struct engine_node *node, void *data)
     }
 
     if (!reevaluate_chassis_features) {
-        return true;
+        return EN_HANDLED_UNCHANGED;
     }
 
     struct chassis_features present_features = config_data->features;
@@ -359,15 +372,15 @@ global_config_sb_chassis_handler(struct engine_node *node, void *data)
     if (chassis_features_changed(&present_features, &config_data->features)) {
         config_data->tracked_data.chassis_features_changed = true;
         config_data->tracked = true;
-        engine_set_node_state(node, EN_UPDATED);
+        return EN_HANDLED_UPDATED;
     }
 
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
 /* generic global config handler for any engine node which has global_config
  * has an input node . */
-bool
+enum engine_input_handler_result
 node_global_config_handler(struct engine_node *node, void *data OVS_UNUSED)
 {
     struct ed_type_global_config *global_config =
@@ -376,13 +389,13 @@ node_global_config_handler(struct engine_node *node, void *data OVS_UNUSED)
     if (!global_config->tracked
         || global_config->tracked_data.chassis_features_changed
         || global_config->tracked_data.nb_options_changed) {
-        return false;
+        return EN_UNHANDLED;
     }
 
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
-bool
+enum engine_input_handler_result
 global_config_nb_logical_switch_handler(struct engine_node *node,
                                         void *data)
 {
@@ -391,8 +404,7 @@ global_config_nb_logical_switch_handler(struct engine_node *node,
         EN_OVSDB_GET(engine_get_input("NB_logical_switch", node));
     const struct nbrec_nb_global *nb = nbrec_nb_global_table_first(
                 EN_OVSDB_GET(engine_get_input("NB_nb_global", node)));
-    const struct sbrec_chassis_table *sbrec_chassis_table =
-        EN_OVSDB_GET(engine_get_input("SB_chassis", node));
+    enum engine_input_handler_result result;
 
     bool ic_vxlan_mode = false;
     const struct nbrec_logical_switch *nbs;
@@ -402,18 +414,17 @@ global_config_nb_logical_switch_handler(struct engine_node *node,
             break;
         }
     }
-    uint32_t max_dp_key =
-        get_ovn_max_dp_key_local(is_vxlan_mode(&nb->options,
-                                               sbrec_chassis_table),
+    config_data->max_dp_tunnel_id =
+        get_ovn_max_dp_key_local(config_data->vxlan_mode,
                                  ic_vxlan_mode);
-    char *max_tunid = xasprintf("%d", max_dp_key);
+    char *max_tunid = xasprintf("%d", config_data->max_dp_tunnel_id);
     struct smap *options = &config_data->nb_options;
     const char *cur_max_tunid = smap_get(options, "max_tunid");
 
     if (!cur_max_tunid || strcmp(max_tunid, cur_max_tunid)) {
-        engine_set_node_state(node, EN_UPDATED);
+        result = EN_HANDLED_UPDATED;
     } else {
-        engine_set_node_state(node, EN_UNCHANGED);
+        result = EN_HANDLED_UNCHANGED;
     }
 
     smap_replace(options, "max_tunid", max_tunid);
@@ -426,7 +437,7 @@ global_config_nb_logical_switch_handler(struct engine_node *node,
 
     config_data->tracked = true;
 
-    return true;
+    return result;
 }
 
 /* static functions. */
@@ -629,11 +640,6 @@ check_nb_options_out_of_sync(
         return true;
     }
 
-    if (config_out_of_sync(&nb->options, &config_data->nb_options,
-                           "vxlan_mode", false)) {
-        return true;
-    }
-
     if (config_out_of_sync(&nb->options, &config_data->nb_options,
                            "always_tunnel", false)) {
         return true;
@@ -695,3 +701,22 @@ chassis_features_changed(const struct chassis_features *present,
 
     return false;
 }
+
+static bool
+is_vxlan_mode(const struct smap *nb_options,
+              const struct sbrec_chassis_table *sbrec_chassis_table)
+{
+    if (!smap_get_bool(nb_options, "vxlan_mode", true)) {
+        return false;
+    }
+
+    const struct sbrec_chassis *chassis;
+    SBREC_CHASSIS_TABLE_FOR_EACH (chassis, sbrec_chassis_table) {
+        for (int i = 0; i < chassis->n_encaps; i++) {
+            if (!strcmp(chassis->encaps[i]->type, "vxlan")) {
+                return true;
+            }
+        }
+    }
+    return false;
+}
