@@ -49,23 +49,30 @@ struct ed_type_global_config {
 
     bool ovn_internal_version_changed;
 
+    bool vxlan_mode;
+    uint32_t max_dp_tunnel_id;
+
     bool tracked;
     struct global_config_tracked_data tracked_data;
 };
 
 void *en_global_config_init(struct engine_node *, struct engine_arg *);
-void en_global_config_run(struct engine_node *, void *data);
+enum engine_node_state en_global_config_run(struct engine_node *, void *data);
 void en_global_config_cleanup(void *data);
 void en_global_config_clear_tracked_data(void *data);
 
-bool global_config_nb_global_handler(struct engine_node *, void *data);
-bool global_config_sb_global_handler(struct engine_node *, void *data);
-bool global_config_sb_chassis_handler(struct engine_node *, void *data);
-bool global_config_nb_logical_switch_handler(struct engine_node *node,
-                                             void *data);
+enum engine_input_handler_result
+global_config_nb_global_handler(struct engine_node *, void *data);
+enum engine_input_handler_result
+global_config_sb_global_handler(struct engine_node *, void *data);
+enum engine_input_handler_result
+global_config_sb_chassis_handler(struct engine_node *, void *data);
+enum engine_input_handler_result
+global_config_nb_logical_switch_handler(struct engine_node *node, void *data);
 
 /* generic global config handler for any engine node which has global_config
  * has an input node . */
-bool node_global_config_handler(struct engine_node *, void *data);
+enum engine_input_handler_result
+node_global_config_handler(struct engine_node *, void *data);
 
 #endif /* EN_GLOBAL_CONFIG_H */
