@@ -62,6 +62,8 @@ lflow_get_input_data(struct engine_node *node,
         engine_get_input_data("ls_stateful", node);
     struct multicast_igmp_data *multicat_igmp_data =
         engine_get_input_data("multicast_igmp", node);
+    struct ic_learned_svc_monitors_data *ic_learned_svc_monitors_data =
+        engine_get_input_data("ic_learned_svc_monitors", node);
 
     lflow_input->sbrec_logical_flow_table =
         EN_OVSDB_GET(engine_get_input("SB_logical_flow", node));
@@ -84,13 +86,18 @@ lflow_get_input_data(struct engine_node *node,
     lflow_input->ls_stateful_table = &ls_stateful_data->table;
     lflow_input->meter_groups = &sync_meters_data->meter_groups;
     lflow_input->lb_datapaths_map = &northd_data->lb_datapaths_map;
-    lflow_input->svc_monitor_map = &northd_data->svc_monitor_map;
+    lflow_input->local_svc_monitors_map =
+        &northd_data->local_svc_monitors_map;
     lflow_input->bfd_ports = &bfd_sync_data->bfd_ports;
     lflow_input->route_data = group_ecmp_route_data;
     lflow_input->route_tables = &routes_data->route_tables;
     lflow_input->route_policies = &route_policies_data->route_policies;
     lflow_input->igmp_groups = &multicat_igmp_data->igmp_groups;
     lflow_input->igmp_lflow_ref = multicat_igmp_data->lflow_ref;
+    lflow_input->ic_learned_svc_monitors_map =
+        &ic_learned_svc_monitors_data->ic_learned_svc_monitors_map;
+    lflow_input->ic_learned_svc_monitors_lflow_ref =
+        ic_learned_svc_monitors_data->lflow_ref;
 
     struct ed_type_global_config *global_config =
         engine_get_input_data("global_config", node);
@@ -104,7 +111,8 @@ lflow_get_input_data(struct engine_node *node,
     lflow_input->sampling_apps = &sampling_app_data->apps;
 }
 
-void en_lflow_run(struct engine_node *node, void *data)
+enum engine_node_state
+en_lflow_run(struct engine_node *node, void *data)
 {
     const struct engine_context *eng_ctx = engine_get_context();
 
@@ -122,16 +130,16 @@ void en_lflow_run(struct engine_node *node, void *data)
                  lflow_data->lflow_table);
     stopwatch_stop(BUILD_LFLOWS_STOPWATCH_NAME, time_msec());
 
-    engine_set_node_state(node, EN_UPDATED);
+    return EN_UPDATED;
 }
 
-bool
+enum engine_input_handler_result
 lflow_northd_handler(struct engine_node *node,
                      void *data)
 {
     struct northd_data *northd_data = engine_get_input_data("northd", node);
     if (!northd_has_tracked_data(&northd_data->trk_data)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     const struct engine_context *eng_ctx = engine_get_context();
@@ -144,44 +152,27 @@ lflow_northd_handler(struct engine_node *node,
                                           &northd_data->trk_data.trk_lsps,
                                           &lflow_input,
                                           lflow_data->lflow_table)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     if (!lflow_handle_northd_lb_changes(
             eng_ctx->ovnsb_idl_txn, &northd_data->trk_data.trk_lbs,
             &lflow_input, lflow_data->lflow_table)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
-    engine_set_node_state(node, EN_UPDATED);
-    return true;
+    return EN_HANDLED_UPDATED;
 }
 
-bool
-lflow_port_group_handler(struct engine_node *node, void *data OVS_UNUSED)
-{
-    struct port_group_data *pg_data =
-        engine_get_input_data("port_group", node);
-
-    /* If the set of switches per port group didn't change then there's no
-     * need to reprocess lflows.  Otherwise, there might be a need to
-     * add/delete port-group ACLs to/from switches. */
-    if (pg_data->ls_port_groups_sets_changed) {
-        return false;
-    }
-
-    engine_set_node_state(node, EN_UPDATED);
-    return true;
-}
-
-bool
+enum engine_input_handler_result
 lflow_lr_stateful_handler(struct engine_node *node, void *data)
 {
     struct ed_type_lr_stateful *lr_sful_data =
         engine_get_input_data("lr_stateful", node);
 
-    if (!lr_stateful_has_tracked_data(&lr_sful_data->trk_data)) {
-        return false;
+    if (!lr_stateful_has_tracked_data(&lr_sful_data->trk_data)
+        || lr_sful_data->trk_data.vip_nats_changed) {
+        return EN_UNHANDLED;
     }
 
     const struct engine_context *eng_ctx = engine_get_context();
@@ -193,21 +184,20 @@ lflow_lr_stateful_handler(struct engine_node *node, void *data)
                                           &lr_sful_data->trk_data,
                                           &lflow_input,
                                           lflow_data->lflow_table)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
-    engine_set_node_state(node, EN_UPDATED);
-    return true;
+    return EN_HANDLED_UPDATED;
 }
 
-bool
+enum engine_input_handler_result
 lflow_ls_stateful_handler(struct engine_node *node, void *data)
 {
     struct ed_type_ls_stateful *ls_sful_data =
         engine_get_input_data("ls_stateful", node);
 
     if (!ls_stateful_has_tracked_data(&ls_sful_data->trk_data)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     const struct engine_context *eng_ctx = engine_get_context();
@@ -219,14 +209,13 @@ lflow_ls_stateful_handler(struct engine_node *node, void *data)
                                           &ls_sful_data->trk_data,
                                           &lflow_input,
                                           lflow_data->lflow_table)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
-    engine_set_node_state(node, EN_UPDATED);
-    return true;
+    return EN_HANDLED_UPDATED;
 }
 
-bool
+enum engine_input_handler_result
 lflow_multicast_igmp_handler(struct engine_node *node, void *data)
 {
     struct multicast_igmp_data *mcast_igmp_data =
@@ -245,7 +234,7 @@ lflow_multicast_igmp_handler(struct engine_node *node, void *data)
                                 lflow_input.ovn_internal_version_changed,
                                 lflow_input.sbrec_logical_flow_table,
                                 lflow_input.sbrec_logical_dp_group_table)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     build_igmp_lflows(&mcast_igmp_data->igmp_groups,
@@ -261,14 +250,13 @@ lflow_multicast_igmp_handler(struct engine_node *node, void *data)
                                lflow_input.ovn_internal_version_changed,
                                lflow_input.sbrec_logical_flow_table,
                                lflow_input.sbrec_logical_dp_group_table)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
-    engine_set_node_state(node, EN_UPDATED);
-    return true;
+    return EN_HANDLED_UPDATED;
 }
 
-bool
+enum engine_input_handler_result
 lflow_group_ecmp_route_change_handler(struct engine_node *node,
                                       void *data OVS_UNUSED)
 {
@@ -277,7 +265,7 @@ lflow_group_ecmp_route_change_handler(struct engine_node *node,
 
     /* If we do not have tracked data we need to recompute. */
     if (!group_ecmp_route_data->tracked) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     const struct engine_context *eng_ctx = engine_get_context();
@@ -304,7 +292,7 @@ lflow_group_ecmp_route_change_handler(struct engine_node *node,
             lflow_input.sbrec_logical_flow_table,
             lflow_input.sbrec_logical_dp_group_table);
         if (!handled) {
-            return false;
+            return EN_UNHANDLED;
         }
     }
 
@@ -326,12 +314,62 @@ lflow_group_ecmp_route_change_handler(struct engine_node *node,
             lflow_input.sbrec_logical_flow_table,
             lflow_input.sbrec_logical_dp_group_table);
         if (!handled) {
-            return false;
+            return EN_UNHANDLED;
         }
     }
 
-    engine_set_node_state(node, EN_UPDATED);
-    return true;
+    return EN_HANDLED_UPDATED;
+}
+
+enum engine_input_handler_result
+lflow_ic_learned_svc_mons_handler(struct engine_node *node,
+                                  void *data)
+{
+    struct ic_learned_svc_monitors_data *ic_learned_svc_monitors_data =
+        engine_get_input_data("ic_learned_svcs", node);
+
+    const struct engine_context *eng_ctx = engine_get_context();
+    struct lflow_data *lflow_data = data;
+    struct lflow_input lflow_input;
+    lflow_get_input_data(node, &lflow_input);
+
+    struct svc_monitors_map_data svc_mons_data =
+        svc_monitors_map_data_init(
+            NULL,
+            &ic_learned_svc_monitors_data->ic_learned_svc_monitors_map,
+            ic_learned_svc_monitors_data->lflow_ref);
+
+    if (!lflow_ref_resync_flows(
+            ic_learned_svc_monitors_data->lflow_ref,
+            lflow_data->lflow_table,
+            eng_ctx->ovnsb_idl_txn,
+            lflow_input.ls_datapaths,
+            lflow_input.lr_datapaths,
+            lflow_input.ovn_internal_version_changed,
+            lflow_input.sbrec_logical_flow_table,
+            lflow_input.sbrec_logical_dp_group_table)) {
+        return EN_UNHANDLED;
+    }
+
+    build_lswitch_arp_nd_ic_learned_svc_mon(
+        &svc_mons_data,
+        lflow_input.ls_ports,
+        lflow_input.svc_monitor_mac,
+        lflow_data->lflow_table);
+
+    if (!lflow_ref_sync_lflows(
+            ic_learned_svc_monitors_data->lflow_ref,
+            lflow_data->lflow_table,
+            eng_ctx->ovnsb_idl_txn,
+            lflow_input.ls_datapaths,
+            lflow_input.lr_datapaths,
+            lflow_input.ovn_internal_version_changed,
+            lflow_input.sbrec_logical_flow_table,
+            lflow_input.sbrec_logical_dp_group_table)) {
+        return EN_UNHANDLED;
+    }
+
+    return EN_HANDLED_UPDATED;
 }
 
 void *en_lflow_init(struct engine_node *node OVS_UNUSED,
