@@ -101,7 +101,7 @@ en_lr_nat_clear_tracked_data(void *data_)
     hmapx_clear(&data->trk_data.crupdated);
 }
 
-void
+enum engine_node_state
 en_lr_nat_run(struct engine_node *node, void *data_)
 {
     struct northd_data *northd_data = engine_get_input_data("northd", node);
@@ -113,20 +113,20 @@ en_lr_nat_run(struct engine_node *node, void *data_)
                        &northd_data->lr_ports);
 
     stopwatch_stop(LR_NAT_RUN_STOPWATCH_NAME, time_msec());
-    engine_set_node_state(node, EN_UPDATED);
+    return EN_UPDATED;
 }
 
 /* Handler functions. */
-bool
+enum engine_input_handler_result
 lr_nat_northd_handler(struct engine_node *node, void *data_)
 {
     struct northd_data *northd_data = engine_get_input_data("northd", node);
     if (!northd_has_tracked_data(&northd_data->trk_data)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     if (!northd_has_lr_nats_in_tracked_data(&northd_data->trk_data)) {
-        return true;
+        return EN_HANDLED_UNCHANGED;
     }
 
     struct ed_type_lr_nat_data *data = data_;
@@ -145,10 +145,10 @@ lr_nat_northd_handler(struct engine_node *node, void *data_)
     }
 
     if (lr_nat_has_tracked_data(&data->trk_data)) {
-        engine_set_node_state(node, EN_UPDATED);
+        return EN_HANDLED_UPDATED;
     }
 
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
 /* static functions. */
@@ -295,13 +295,15 @@ lr_nat_entry_set_dgw_port(const struct ovn_datapath *od,
     /* Validate gateway_port of NAT rule. */
     nat_entry->l3dgw_port = NULL;
     if (nat->gateway_port == NULL) {
-        if (od->n_l3dgw_ports == 1) {
-            nat_entry->l3dgw_port = od->l3dgw_ports[0];
-        } else if (od->n_l3dgw_ports > 1) {
+        if (vector_len(&od->l3dgw_ports) == 1) {
+            nat_entry->l3dgw_port = vector_get(&od->l3dgw_ports, 0,
+                                               struct ovn_port *);
+        } else if (vector_len(&od->l3dgw_ports) > 1) {
             /* Find the DGP reachable for the NAT external IP. */
-            for (size_t i = 0; i < od->n_l3dgw_ports; i++) {
-               if (lrp_find_member_ip(od->l3dgw_ports[i], nat->external_ip)) {
-                   nat_entry->l3dgw_port = od->l3dgw_ports[i];
+            struct ovn_port *dgp;
+            VECTOR_FOR_EACH (&od->l3dgw_ports, dgp) {
+               if (lrp_find_member_ip(dgp, nat->external_ip)) {
+                   nat_entry->l3dgw_port = dgp;
                    break;
                }
             }
@@ -342,7 +344,8 @@ lr_nat_entry_set_dgw_port(const struct ovn_datapath *od,
         return true;
     }
 
-    if (od->n_l3dgw_ports && nat_entry->type == DNAT_AND_SNAT &&
+    if (!vector_is_empty(&od->l3dgw_ports) &&
+        nat_entry->type == DNAT_AND_SNAT &&
         nat->logical_port && nat->external_mac) {
             nat_entry->is_distributed = true;
     }
@@ -359,6 +362,10 @@ lr_nat_record_init(struct lr_nat_record *lrnat_rec,
     lrnat_rec->nbr_uuid = od->nbr->header_.uuid;
 
     shash_init(&lrnat_rec->snat_ips);
+    sset_init(&lrnat_rec->external_ips);
+    for (size_t i = 0; i < od->nbr->n_nat; i++) {
+        sset_add(&lrnat_rec->external_ips, od->nbr->nat[i]->external_ip);
+    }
 
     sset_init(&lrnat_rec->external_macs);
     lrnat_rec->has_distributed_nat = false;
@@ -528,6 +535,7 @@ lr_nat_record_clear(struct lr_nat_record *lrnat_rec)
     }
 
     free(lrnat_rec->nat_entries);
+    sset_destroy(&lrnat_rec->external_ips);
     sset_destroy(&lrnat_rec->external_macs);
 }
 
