@@ -80,6 +80,9 @@ struct lr_nat_record {
 
     bool has_distributed_nat;
 
+    /* Set of nat external ips on the router. */
+    struct sset external_ips;
+
     /* Set of nat external macs on the router. */
     struct sset external_macs;
 
@@ -118,10 +121,10 @@ struct ed_type_lr_nat_data {
 void *en_lr_nat_init(struct engine_node *, struct engine_arg *);
 void en_lr_nat_cleanup(void *data);
 void en_lr_nat_clear_tracked_data(void *data);
-void en_lr_nat_run(struct engine_node *, void *data);
+enum engine_node_state en_lr_nat_run(struct engine_node *, void *data);
 
-bool lr_nat_logical_router_handler(struct engine_node *, void *data);
-bool lr_nat_northd_handler(struct engine_node *, void *data);
+enum engine_input_handler_result lr_nat_northd_handler(struct engine_node *,
+                                                       void *data);
 
 static inline bool
 nat_entry_is_v6(const struct ovn_nat *nat_entry)
@@ -134,4 +137,4 @@ lr_nat_has_tracked_data(struct lr_nat_tracked_data *trk_data) {
     return !hmapx_is_empty(&trk_data->crupdated);
 }
 
-#endif /* EN_LR_NAT_H */
\ No newline at end of file
+#endif /* EN_LR_NAT_H */
