@@ -65,20 +65,13 @@ static void ls_stateful_record_destroy(struct ls_stateful_record *);
 static void ls_stateful_record_init(
     struct ls_stateful_record *,
     const struct ovn_datapath *,
-    const struct ls_port_group *,
-    const struct ls_port_group_table *);
-static void ls_stateful_record_reinit(
-    struct ls_stateful_record *,
-    const struct ovn_datapath *,
-    const struct ls_port_group *,
     const struct ls_port_group_table *);
 static bool ls_has_lb_vip(const struct ovn_datapath *);
-static void ls_stateful_record_set_acl_flags(
-    struct ls_stateful_record *, const struct ovn_datapath *,
-    const struct ls_port_group *, const struct ls_port_group_table *);
-static bool ls_stateful_record_set_acl_flags_(struct ls_stateful_record *,
-                                              struct nbrec_acl **,
-                                              size_t n_acls);
+static void ls_stateful_record_set_acls(
+    struct ls_stateful_record *, const struct nbrec_logical_switch *,
+    const struct ls_port_group_table *);
+static void ls_stateful_record_set_acls_(struct ls_stateful_record *,
+                                         struct nbrec_acl **, size_t n_acls);
 static struct ls_stateful_input ls_stateful_get_input_data(
     struct engine_node *);
 
@@ -113,7 +106,7 @@ en_ls_stateful_clear_tracked_data(void *data_)
     hmapx_clear(&data->trk_data.crupdated);
 }
 
-void
+enum engine_node_state
 en_ls_stateful_run(struct engine_node *node, void *data_)
 {
     struct ls_stateful_input input_data = ls_stateful_get_input_data(node);
@@ -126,21 +119,21 @@ en_ls_stateful_run(struct engine_node *node, void *data_)
                           input_data.ls_port_groups);
 
     stopwatch_stop(LS_STATEFUL_RUN_STOPWATCH_NAME, time_msec());
-    engine_set_node_state(node, EN_UPDATED);
+    return EN_UPDATED;
 }
 
 /* Handler functions. */
-bool
+enum engine_input_handler_result
 ls_stateful_northd_handler(struct engine_node *node, void *data_)
 {
     struct northd_data *northd_data = engine_get_input_data("northd", node);
     if (!northd_has_tracked_data(&northd_data->trk_data)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     if (!northd_has_ls_lbs_in_tracked_data(&northd_data->trk_data) &&
         !northd_has_ls_acls_in_tracked_data(&northd_data->trk_data)) {
-        return true;
+        return EN_HANDLED_UNCHANGED;
     }
 
     struct northd_tracked_data *nd_changes = &northd_data->trk_data;
@@ -148,91 +141,97 @@ ls_stateful_northd_handler(struct engine_node *node, void *data_)
     struct ed_type_ls_stateful *data = data_;
     struct hmapx_node *hmapx_node;
 
-    struct hmapx changed_stateful_od = HMAPX_INITIALIZER(&changed_stateful_od);
     HMAPX_FOR_EACH (hmapx_node, &nd_changes->ls_with_changed_lbs) {
-        hmapx_add(&changed_stateful_od, hmapx_node->data);
-    }
-
-    HMAPX_FOR_EACH (hmapx_node, &nd_changes->ls_with_changed_acls) {
-        hmapx_add(&changed_stateful_od, hmapx_node->data);
-    }
-
-    HMAPX_FOR_EACH (hmapx_node, &changed_stateful_od) {
         const struct ovn_datapath *od = hmapx_node->data;
 
-        struct ls_stateful_record *ls_stateful_rec = ls_stateful_table_find_(
-            &data->table, od->nbs);
+        struct ls_stateful_record *ls_stateful_rec =
+            ls_stateful_table_find_(&data->table, od->nbs);
         ovs_assert(ls_stateful_rec);
-        ls_stateful_record_reinit(ls_stateful_rec, od, NULL,
-                                  input_data.ls_port_groups);
+        ls_stateful_rec->has_lb_vip = ls_has_lb_vip(od);
 
         /* Add the ls_stateful_rec to the tracking data. */
         hmapx_add(&data->trk_data.crupdated, ls_stateful_rec);
     }
 
-    if (ls_stateful_has_tracked_data(&data->trk_data)) {
-        engine_set_node_state(node, EN_UPDATED);
+    HMAPX_FOR_EACH (hmapx_node, &nd_changes->ls_with_changed_acls) {
+        const struct ovn_datapath *od = hmapx_node->data;
+
+        struct ls_stateful_record *ls_stateful_rec =
+            ls_stateful_table_find_(&data->table, od->nbs);
+        ovs_assert(ls_stateful_rec);
+        /* Ensure that only one handler per engine run calls
+         * ls_stateful_record_set_acls on the same ls_stateful_rec by
+         * calling it only when the ls_stateful_rec is added to the hmapx. */
+        if (hmapx_add(&data->trk_data.crupdated, ls_stateful_rec)) {
+            ls_stateful_record_set_acls(ls_stateful_rec, od->nbs,
+                                         input_data.ls_port_groups);
+        }
     }
 
-    hmapx_destroy(&changed_stateful_od);
+    if (ls_stateful_has_tracked_data(&data->trk_data)) {
+        return EN_HANDLED_UPDATED;
+    }
 
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
-bool
+enum engine_input_handler_result
 ls_stateful_port_group_handler(struct engine_node *node, void *data_)
 {
     struct port_group_data *pg_data =
         engine_get_input_data("port_group", node);
 
-    if (pg_data->ls_port_groups_sets_changed) {
-        return false;
-    }
-
-    /* port_group engine node doesn't provide the tracking data yet.
-     * Loop through all the ls port groups and update the ls_stateful_rec.
-     * This is still better than returning false. */
-    struct ls_stateful_input input_data = ls_stateful_get_input_data(node);
     struct ed_type_ls_stateful *data = data_;
-    const struct ls_port_group *ls_pg;
-
-    LS_PORT_GROUP_TABLE_FOR_EACH (ls_pg, input_data.ls_port_groups) {
+    struct hmapx_node *hmap_node;
+    HMAPX_FOR_EACH (hmap_node, &pg_data->ls_port_groups_sets_changed) {
+        const struct nbrec_logical_switch *nbs = hmap_node->data;
         struct ls_stateful_record *ls_stateful_rec =
-            ls_stateful_table_find_(&data->table, ls_pg->nbs);
+            ls_stateful_table_find_(&data->table, nbs);
         ovs_assert(ls_stateful_rec);
-        const struct ovn_datapath *od =
-            ovn_datapaths_find_by_index(input_data.ls_datapaths,
-                                        ls_stateful_rec->ls_index);
-        bool had_stateful_acl = ls_stateful_rec->has_stateful_acl;
-        struct acl_tier old_max = ls_stateful_rec->max_acl_tier;
-        bool had_acls = ls_stateful_rec->has_acls;
-        bool modified = false;
-
-        ls_stateful_record_reinit(ls_stateful_rec, od, ls_pg,
-                                  input_data.ls_port_groups);
-
-        struct acl_tier new_max = ls_stateful_rec->max_acl_tier;
-
-        /* Using memcmp for struct acl_tier is fine since there is no padding
-         * in the struct. However, if the structure is changed, the memcmp
-         * may need to be updated to compare individual struct fields.
-         */
-        if ((had_stateful_acl != ls_stateful_rec->has_stateful_acl)
-            || (had_acls != ls_stateful_rec->has_acls)
-            || memcmp(&old_max, &new_max, sizeof(old_max))) {
-            modified = true;
-        }
-
-        if (modified) {
-            /* Add the ls_stateful_rec to the tracking data. */
-            hmapx_add(&data->trk_data.crupdated, ls_stateful_rec);
+        /* Ensure that only one handler per engine run calls
+         * ls_stateful_record_set_acls on the same ls_stateful_rec by
+         * calling it only when the ls_stateful_rec is added to the hmapx.*/
+        if (hmapx_add(&data->trk_data.crupdated, ls_stateful_rec)) {
+            ls_stateful_record_set_acls(ls_stateful_rec,
+                                        nbs,
+                                        &pg_data->ls_port_groups);
         }
     }
 
     if (ls_stateful_has_tracked_data(&data->trk_data)) {
-        engine_set_node_state(node, EN_UPDATED);
+        return EN_HANDLED_UPDATED;
     }
-    return true;
+    return EN_HANDLED_UNCHANGED;
+}
+
+enum engine_input_handler_result
+ls_stateful_acl_handler(struct engine_node *node, void *data_)
+{
+    struct ed_type_ls_stateful *data = data_;
+    const struct nbrec_acl_table *nbrec_acl_table =
+        EN_OVSDB_GET(engine_get_input("NB_acl", node));
+
+    const struct nbrec_acl *acl;
+    NBREC_ACL_TABLE_FOR_EACH_TRACKED (acl, nbrec_acl_table) {
+        /* The creation and deletion is handled in relation to LS/PG rather
+         * than the ACL itself. */
+        if (nbrec_acl_is_new(acl) || nbrec_acl_is_deleted(acl)) {
+            continue;
+        }
+
+        struct ls_stateful_record *ls_stateful_rec;
+        LS_STATEFUL_TABLE_FOR_EACH (ls_stateful_rec, &data->table) {
+            if (uuidset_contains(&ls_stateful_rec->related_acls,
+                                 &acl->header_.uuid)) {
+                hmapx_add(&data->trk_data.crupdated, ls_stateful_rec);
+            }
+        }
+    }
+
+    if (ls_stateful_has_tracked_data(&data->trk_data)) {
+        return EN_HANDLED_UPDATED;
+    }
+    return EN_HANDLED_UNCHANGED;
 }
 
 /* static functions. */
@@ -295,7 +294,8 @@ ls_stateful_record_create(struct ls_stateful_table *table,
         xzalloc(sizeof *ls_stateful_rec);
     ls_stateful_rec->ls_index = od->index;
     ls_stateful_rec->nbs_uuid = od->nbs->header_.uuid;
-    ls_stateful_record_init(ls_stateful_rec, od, NULL, ls_pgs);
+    uuidset_init(&ls_stateful_rec->related_acls);
+    ls_stateful_record_init(ls_stateful_rec, od, ls_pgs);
     ls_stateful_rec->lflow_ref = lflow_ref_create();
 
     hmap_insert(&table->entries, &ls_stateful_rec->key_node,
@@ -307,6 +307,7 @@ ls_stateful_record_create(struct ls_stateful_table *table,
 static void
 ls_stateful_record_destroy(struct ls_stateful_record *ls_stateful_rec)
 {
+    uuidset_destroy(&ls_stateful_rec->related_acls);
     lflow_ref_destroy(ls_stateful_rec->lflow_ref);
     free(ls_stateful_rec);
 }
@@ -314,20 +315,10 @@ ls_stateful_record_destroy(struct ls_stateful_record *ls_stateful_rec)
 static void
 ls_stateful_record_init(struct ls_stateful_record *ls_stateful_rec,
                       const struct ovn_datapath *od,
-                      const struct ls_port_group *ls_pg,
                       const struct ls_port_group_table *ls_pgs)
 {
     ls_stateful_rec->has_lb_vip = ls_has_lb_vip(od);
-    ls_stateful_record_set_acl_flags(ls_stateful_rec, od, ls_pg, ls_pgs);
-}
-
-static void
-ls_stateful_record_reinit(struct ls_stateful_record *ls_stateful_rec,
-                          const struct ovn_datapath *od,
-                          const struct ls_port_group *ls_pg,
-                          const struct ls_port_group_table *ls_pgs)
-{
-    ls_stateful_record_init(ls_stateful_rec, od, ls_pg, ls_pgs);
+    ls_stateful_record_set_acls(ls_stateful_rec, od->nbs, ls_pgs);
 }
 
 static bool
@@ -365,36 +356,28 @@ ls_has_lb_vip(const struct ovn_datapath *od)
 }
 
 static void
-ls_stateful_record_set_acl_flags(struct ls_stateful_record *ls_stateful_rec,
-                                 const struct ovn_datapath *od,
-                                 const struct ls_port_group *ls_pg,
-                                 const struct ls_port_group_table *ls_pgs)
+ls_stateful_record_set_acls(struct ls_stateful_record *ls_stateful_rec,
+                            const struct nbrec_logical_switch *nbs,
+                            const struct ls_port_group_table *ls_pgs)
 {
     ls_stateful_rec->has_stateful_acl = false;
     memset(&ls_stateful_rec->max_acl_tier, 0,
            sizeof ls_stateful_rec->max_acl_tier);
     ls_stateful_rec->has_acls = false;
+    uuidset_clear(&ls_stateful_rec->related_acls);
 
-    if (ls_stateful_record_set_acl_flags_(ls_stateful_rec, od->nbs->acls,
-                                          od->nbs->n_acls)) {
-        return;
-    }
-
-    if (!ls_pg) {
-        ls_pg = ls_port_group_table_find(ls_pgs, od->nbs);
-    }
+    ls_stateful_record_set_acls_(ls_stateful_rec, nbs->acls,
+                                 nbs->n_acls);
 
+    struct ls_port_group *ls_pg = ls_port_group_table_find(ls_pgs, nbs);
     if (!ls_pg) {
         return;
     }
 
     const struct ls_port_group_record *ls_pg_rec;
     HMAP_FOR_EACH (ls_pg_rec, key_node, &ls_pg->nb_pgs) {
-        if (ls_stateful_record_set_acl_flags_(ls_stateful_rec,
-                                              ls_pg_rec->nb_pg->acls,
-                                              ls_pg_rec->nb_pg->n_acls)) {
-            return;
-        }
+        ls_stateful_record_set_acls_(ls_stateful_rec, ls_pg_rec->nb_pg->acls,
+                                     ls_pg_rec->nb_pg->n_acls);
     }
 }
 
@@ -421,46 +404,24 @@ update_ls_max_acl_tier(struct ls_stateful_record *ls_stateful_rec,
     *tier = MAX(*tier, acl->tier);
 }
 
-static bool
-ls_acl_tiers_are_maxed_out(struct acl_tier *acl_tier,
-                           uint64_t max_allowed_acl_tier)
+static void
+ls_stateful_record_set_acls_(struct ls_stateful_record *ls_stateful_rec,
+                             struct nbrec_acl **acls, size_t n_acls)
 {
-    return acl_tier->ingress_post_lb == max_allowed_acl_tier &&
-           acl_tier->ingress_pre_lb == max_allowed_acl_tier &&
-           acl_tier->egress == max_allowed_acl_tier;
-}
-
-static bool
-ls_stateful_record_set_acl_flags_(struct ls_stateful_record *ls_stateful_rec,
-                                  struct nbrec_acl **acls,
-                                  size_t n_acls)
-{
-    /* A true return indicates that there are no possible ACL flags
-     * left to set on ls_stateful record. A false return indicates that
-     * further ACLs should be explored in case more flags need to be
-     * set on ls_stateful record.
-     */
     if (!n_acls) {
-        return false;
+        return;
     }
 
     ls_stateful_rec->has_acls = true;
     for (size_t i = 0; i < n_acls; i++) {
         const struct nbrec_acl *acl = acls[i];
         update_ls_max_acl_tier(ls_stateful_rec, acl);
+        uuidset_insert(&ls_stateful_rec->related_acls, &acl->header_.uuid);
         if (!ls_stateful_rec->has_stateful_acl
                 && !strcmp(acl->action, "allow-related")) {
             ls_stateful_rec->has_stateful_acl = true;
         }
-        if (ls_stateful_rec->has_stateful_acl &&
-            ls_acl_tiers_are_maxed_out(
-                &ls_stateful_rec->max_acl_tier,
-                nbrec_acl_col_tier.type.key.integer.max)) {
-            return true;
-        }
     }
-
-    return false;
 }
 
 static struct ls_stateful_input
