@@ -41,6 +41,10 @@ static void
 northd_get_input_data(struct engine_node *node,
                       struct northd_input *input_data)
 {
+    input_data->nbrec_mirror_by_type_and_sink =
+        engine_ovsdb_node_get_index(
+            engine_get_input("NB_mirror", node),
+            "nbrec_mirror_by_type_and_sink");
     input_data->sbrec_chassis_by_name =
         engine_ovsdb_node_get_index(
             engine_get_input("SB_chassis", node),
@@ -61,20 +65,26 @@ northd_get_input_data(struct engine_node *node,
         engine_ovsdb_node_get_index(
             engine_get_input("SB_fdb", node),
             "sbrec_fdb_by_dp_and_port");
+    input_data->sbrec_service_monitor_by_learned_type =
+        engine_ovsdb_node_get_index(
+            engine_get_input("SB_service_monitor", node),
+            "sbrec_service_monitor_by_learned_type");
 
-    input_data->nbrec_logical_switch_table =
-        EN_OVSDB_GET(engine_get_input("NB_logical_switch", node));
-    input_data->nbrec_logical_router_table =
-        EN_OVSDB_GET(engine_get_input("NB_logical_router", node));
     input_data->nbrec_static_mac_binding_table =
         EN_OVSDB_GET(engine_get_input("NB_static_mac_binding", node));
     input_data->nbrec_chassis_template_var_table =
         EN_OVSDB_GET(engine_get_input("NB_chassis_template_var", node));
     input_data->nbrec_mirror_table =
         EN_OVSDB_GET(engine_get_input("NB_mirror", node));
+    input_data->nbrec_mirror_rule_table =
+        EN_OVSDB_GET(engine_get_input("NB_mirror_rule", node));
+    input_data->nbrec_port_group_table =
+        EN_OVSDB_GET(engine_get_input("NB_port_group", node));
+    input_data->sbrec_service_monitor_by_learned_type =
+        engine_ovsdb_node_get_index(
+            engine_get_input("SB_service_monitor", node),
+            "sbrec_service_monitor_by_learned_type");
 
-    input_data->sbrec_datapath_binding_table =
-        EN_OVSDB_GET(engine_get_input("SB_datapath_binding", node));
     input_data->sbrec_port_binding_table =
         EN_OVSDB_GET(engine_get_input("SB_port_binding", node));
     input_data->sbrec_mac_binding_table =
@@ -110,9 +120,22 @@ northd_get_input_data(struct engine_node *node,
     input_data->svc_monitor_mac = global_config->svc_monitor_mac;
     input_data->svc_monitor_mac_ea = global_config->svc_monitor_mac_ea;
     input_data->features = &global_config->features;
+    input_data->vxlan_mode = global_config->vxlan_mode;
+
+    input_data->synced_lses =
+        engine_get_input_data("datapath_synced_logical_switch", node);
+
+    input_data->synced_lrs =
+        engine_get_input_data("datapath_synced_logical_router", node);
+
+    /* Add Service Monitor data for interconnect learned records. */
+    struct ic_learned_svc_monitors_data *ic_learned_svc_monitors_data =
+        engine_get_input_data("ic_learned_svc_monitors", node);
+    input_data->ic_learned_svc_monitors_map =
+        &ic_learned_svc_monitors_data->ic_learned_svc_monitors_map;
 }
 
-void
+enum engine_node_state
 en_northd_run(struct engine_node *node, void *data)
 {
     const struct engine_context *eng_ctx = engine_get_context();
@@ -126,13 +149,12 @@ en_northd_run(struct engine_node *node, void *data)
 
     COVERAGE_INC(northd_run);
     stopwatch_start(OVNNB_DB_RUN_STOPWATCH_NAME, time_msec());
-    ovnnb_db_run(&input_data, data, eng_ctx->ovnnb_idl_txn,
-                 eng_ctx->ovnsb_idl_txn);
+    ovnnb_db_run(&input_data, data, eng_ctx->ovnsb_idl_txn);
     stopwatch_stop(OVNNB_DB_RUN_STOPWATCH_NAME, time_msec());
-    engine_set_node_state(node, EN_UPDATED);
+    return EN_UPDATED;
 }
 
-bool
+enum engine_input_handler_result
 northd_nb_logical_switch_handler(struct engine_node *node,
                                  void *data)
 {
@@ -144,17 +166,18 @@ northd_nb_logical_switch_handler(struct engine_node *node,
     northd_get_input_data(node, &input_data);
 
     if (!northd_handle_ls_changes(eng_ctx->ovnsb_idl_txn, &input_data, nd)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
-    if (northd_has_tracked_data(&nd->trk_data)) {
-        engine_set_node_state(node, EN_UPDATED);
+    bool ipam_update = northd_handle_ipam_changes(nd);
+    if (northd_has_tracked_data(&nd->trk_data) || ipam_update) {
+        return EN_HANDLED_UPDATED;
     }
 
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
-bool
+enum engine_input_handler_result
 northd_sb_port_binding_handler(struct engine_node *node,
                                void *data)
 {
@@ -166,13 +189,13 @@ northd_sb_port_binding_handler(struct engine_node *node,
 
     if (!northd_handle_sb_port_binding_changes(
         input_data.sbrec_port_binding_table, &nd->ls_ports, &nd->lr_ports)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
-bool
+enum engine_input_handler_result
 northd_nb_logical_router_handler(struct engine_node *node,
                                  void *data)
 {
@@ -182,23 +205,23 @@ northd_nb_logical_router_handler(struct engine_node *node,
     northd_get_input_data(node, &input_data);
 
     if (!northd_handle_lr_changes(&input_data, nd)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     if (northd_has_lr_nats_in_tracked_data(&nd->trk_data)) {
-        engine_set_node_state(node, EN_UPDATED);
+        return EN_HANDLED_UPDATED;
     }
 
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
-bool
+enum engine_input_handler_result
 northd_lb_data_handler(struct engine_node *node, void *data)
 {
     struct ed_type_lb_data *lb_data = engine_get_input_data("lb_data", node);
 
     if (!lb_data->tracked) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     struct northd_data *nd = data;
@@ -208,17 +231,17 @@ northd_lb_data_handler(struct engine_node *node, void *data)
                                        &nd->lb_datapaths_map,
                                        &nd->lb_group_datapaths_map,
                                        &nd->trk_data)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     if (northd_has_lbs_in_tracked_data(&nd->trk_data)) {
-        engine_set_node_state(node, EN_UPDATED);
+        return EN_HANDLED_UPDATED;
     }
 
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
-bool
+enum engine_input_handler_result
 northd_global_config_handler(struct engine_node *node, void *data OVS_UNUSED)
 {
     struct ed_type_global_config *global_config =
@@ -226,19 +249,41 @@ northd_global_config_handler(struct engine_node *node, void *data OVS_UNUSED)
 
     if (!global_config->tracked
         || global_config->tracked_data.nb_options_changed) {
-        return false;
+        return EN_UNHANDLED;
     }
 
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
-bool
+enum engine_input_handler_result
+northd_nb_port_group_handler(struct engine_node *node, void *data)
+{
+    struct northd_data *nd = data;
+
+    struct northd_input input_data;
+    northd_get_input_data(node, &input_data);
+
+    /* This handler cares only about ACLs, the port group itself has separate
+     * node. */
+    if (!northd_handle_pgs_acl_changes(&input_data, nd)) {
+        return EN_UNHANDLED;
+    }
+
+    if (northd_has_tracked_data(&nd->trk_data)) {
+        return EN_HANDLED_UPDATED;
+    }
+
+    return EN_HANDLED_UNCHANGED;
+}
+
+
+enum engine_input_handler_result
 route_policies_northd_change_handler(struct engine_node *node,
                                      void *data OVS_UNUSED)
 {
     struct northd_data *northd_data = engine_get_input_data("northd", node);
     if (!northd_has_tracked_data(&northd_data->trk_data)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     /* This node uses the below data from the en_northd engine node.
@@ -258,10 +303,10 @@ route_policies_northd_change_handler(struct engine_node *node,
      *      Note: When we add I-P to handle route policies changes, we need
      *      to revisit this handler.
      */
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
-void
+enum engine_node_state
 en_route_policies_run(struct engine_node *node, void *data)
 {
     struct northd_data *northd_data = engine_get_input_data("northd", node);
@@ -280,16 +325,16 @@ en_route_policies_run(struct engine_node *node, void *data)
                              &route_policies_data->chain_ids);
     }
 
-    engine_set_node_state(node, EN_UPDATED);
+    return EN_UPDATED;
 }
 
-bool
+enum engine_input_handler_result
 routes_northd_change_handler(struct engine_node *node,
                                     void *data OVS_UNUSED)
 {
     struct northd_data *northd_data = engine_get_input_data("northd", node);
     if (!northd_has_tracked_data(&northd_data->trk_data)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     /* This node uses the below data from the en_northd engine node.
@@ -309,10 +354,10 @@ routes_northd_change_handler(struct engine_node *node,
      *      Note: When we add I-P to handle static routes changes, we need
      *      to revisit this handler.
      */
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
-void
+enum engine_node_state
 en_routes_run(struct engine_node *node, void *data)
 {
     struct northd_data *northd_data = engine_get_input_data("northd", node);
@@ -338,10 +383,10 @@ en_routes_run(struct engine_node *node, void *data)
                             &routes_data->bfd_active_connections);
     }
 
-    engine_set_node_state(node, EN_UPDATED);
+    return EN_UPDATED;
 }
 
-void
+enum engine_node_state
 en_bfd_run(struct engine_node *node, void *data)
 {
     struct bfd_data *bfd_data = data;
@@ -354,15 +399,15 @@ en_bfd_run(struct engine_node *node, void *data)
     bfd_init(data);
     build_bfd_map(nbrec_bfd_table, sbrec_bfd_table,
                   &bfd_data->bfd_connections);
-    engine_set_node_state(node, EN_UPDATED);
+    return EN_UPDATED;
 }
 
-bool
+enum engine_input_handler_result
 bfd_sync_northd_change_handler(struct engine_node *node, void *data OVS_UNUSED)
 {
     struct northd_data *northd_data = engine_get_input_data("northd", node);
     if (!northd_has_tracked_data(&northd_data->trk_data)) {
-        return false;
+        return EN_UNHANDLED;
     }
 
     /* This node uses the below data from the en_northd engine node.
@@ -374,10 +419,10 @@ bfd_sync_northd_change_handler(struct engine_node *node, void *data OVS_UNUSED)
      *      Note: When we add I-P to the created/deleted logical router ports,
      *      we need to revisit this handler.
      */
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
-void
+enum engine_node_state
 en_bfd_sync_run(struct engine_node *node, void *data)
 {
     struct northd_data *northd_data = engine_get_input_data("northd", node);
@@ -391,14 +436,39 @@ en_bfd_sync_run(struct engine_node *node, void *data)
         EN_OVSDB_GET(engine_get_input("NB_bfd", node));
     struct bfd_sync_data *bfd_sync_data = data;
 
-    bfd_sync_destroy(data);
-    bfd_sync_init(data);
+    struct sset new_bfd_ports = SSET_INITIALIZER(&new_bfd_ports);
     bfd_table_sync(eng_ctx->ovnsb_idl_txn, nbrec_bfd_table,
                    &northd_data->lr_ports, &bfd_data->bfd_connections,
                    &route_policies_data->bfd_active_connections,
                    &routes_data->bfd_active_connections,
-                   &bfd_sync_data->bfd_ports);
-    engine_set_node_state(node, EN_UPDATED);
+                   &new_bfd_ports);
+
+    enum engine_node_state new_state =
+        sset_equals(&new_bfd_ports, &bfd_sync_data->bfd_ports)
+        ? EN_UNCHANGED : EN_UPDATED;
+
+    bfd_sync_swap(bfd_sync_data, &new_bfd_ports);
+    sset_destroy(&new_bfd_ports);
+    return new_state;
+}
+
+enum engine_node_state
+en_ic_learned_svc_monitors_run(struct engine_node *node, void *data_)
+{
+    struct ic_learned_svc_monitors_data *data = data_;
+    struct ovsdb_idl_index *sbrec_service_monitor_by_learned_type =
+        engine_ovsdb_node_get_index(
+            engine_get_input("SB_service_monitor", node),
+            "sbrec_service_monitor_by_learned_type");
+
+    ic_learned_svc_monitors_cleanup(data);
+    ic_learned_svc_monitors_init(data);
+
+    build_ic_learned_svc_monitors_map(
+        &data->ic_learned_svc_monitors_map,
+        sbrec_service_monitor_by_learned_type);
+
+    return EN_UPDATED;
 }
 
 void
@@ -451,6 +521,15 @@ void
     return data;
 }
 
+void
+*en_ic_learned_svc_monitors_init(struct engine_node *node OVS_UNUSED,
+                                 struct engine_arg *arg OVS_UNUSED)
+{
+    struct ic_learned_svc_monitors_data *data = xzalloc(sizeof *data);
+    ic_learned_svc_monitors_init(data);
+    return data;
+}
+
 void
 en_northd_cleanup(void *data)
 {
@@ -464,7 +543,7 @@ en_northd_clear_tracked_data(void *data_)
     destroy_northd_data_tracked_changes(data);
 }
 
-bool
+enum engine_input_handler_result
 northd_sb_fdb_change_handler(struct engine_node *node, void *data)
 {
     struct northd_data *nd = data;
@@ -497,7 +576,7 @@ northd_sb_fdb_change_handler(struct engine_node *node, void *data)
         sbrec_fdb_delete(fdb_prev_del);
     }
 
-    return true;
+    return EN_HANDLED_UNCHANGED;
 }
 
 void
@@ -523,3 +602,9 @@ en_bfd_sync_cleanup(void *data)
 {
     bfd_sync_destroy(data);
 }
+
+void
+en_ic_learned_svc_monitors_cleanup(void *data)
+{
+    ic_learned_svc_monitors_cleanup(data);
+}
