@@ -104,14 +104,15 @@ struct port_group_input {
 struct port_group_data {
     struct ls_port_group_table ls_port_groups;
     struct port_group_ls_table port_groups_lses;
-    bool ls_port_groups_sets_changed;
+    struct hmapx ls_port_groups_sets_changed;
 };
 
 void *en_port_group_init(struct engine_node *, struct engine_arg *);
 void en_port_group_cleanup(void *data);
 void en_port_group_clear_tracked_data(void *data);
-void en_port_group_run(struct engine_node *, void *data);
+enum engine_node_state en_port_group_run(struct engine_node *, void *data);
 
-bool port_group_nb_port_group_handler(struct engine_node *, void *data);
+enum engine_input_handler_result
+port_group_nb_port_group_handler(struct engine_node *, void *data);
 
 #endif /* EN_PORT_GROUP_H */
