@@ -47,6 +47,9 @@
 #include "en-advertised-route-sync.h"
 #include "en-learned-route-sync.h"
 #include "en-group-ecmp-route.h"
+#include "en-datapath-logical-router.h"
+#include "en-datapath-logical-switch.h"
+#include "en-datapath-sync.h"
 #include "unixctl.h"
 #include "util.h"
 
@@ -55,23 +58,24 @@ VLOG_DEFINE_THIS_MODULE(inc_proc_northd);
 static unixctl_cb_func chassis_features_list;
 
 #define NB_NODES \
-    NB_NODE(nb_global, "nb_global") \
-    NB_NODE(logical_switch, "logical_switch") \
-    NB_NODE(address_set, "address_set") \
-    NB_NODE(port_group, "port_group") \
-    NB_NODE(load_balancer, "load_balancer") \
-    NB_NODE(load_balancer_group, "load_balancer_group") \
-    NB_NODE(acl, "acl") \
-    NB_NODE(logical_router, "logical_router") \
-    NB_NODE(mirror, "mirror") \
-    NB_NODE(meter, "meter") \
-    NB_NODE(bfd, "bfd") \
-    NB_NODE(static_mac_binding, "static_mac_binding") \
-    NB_NODE(chassis_template_var, "chassis_template_var") \
-    NB_NODE(sampling_app, "sampling_app")
+    NB_NODE(nb_global) \
+    NB_NODE(logical_switch) \
+    NB_NODE(address_set) \
+    NB_NODE(port_group) \
+    NB_NODE(load_balancer) \
+    NB_NODE(load_balancer_group) \
+    NB_NODE(acl) \
+    NB_NODE(logical_router) \
+    NB_NODE(mirror) \
+    NB_NODE(mirror_rule) \
+    NB_NODE(meter) \
+    NB_NODE(bfd) \
+    NB_NODE(static_mac_binding) \
+    NB_NODE(chassis_template_var) \
+    NB_NODE(sampling_app)
 
     enum nb_engine_node {
-#define NB_NODE(NAME, NAME_STR) NB_##NAME,
+#define NB_NODE(NAME) NB_##NAME,
     NB_NODES
 #undef NB_NODE
     };
@@ -82,40 +86,40 @@ static unixctl_cb_func chassis_features_list;
  * en_nb_<TABLE_NAME>_init()
  * en_nb_<TABLE_NAME>_cleanup()
  */
-#define NB_NODE(NAME, NAME_STR) ENGINE_FUNC_NB(NAME);
+#define NB_NODE(NAME) ENGINE_FUNC_NB(NAME);
     NB_NODES
 #undef NB_NODE
 
 #define SB_NODES \
-    SB_NODE(sb_global, "sb_global") \
-    SB_NODE(chassis, "chassis") \
-    SB_NODE(address_set, "address_set") \
-    SB_NODE(port_group, "port_group") \
-    SB_NODE(logical_flow, "logical_flow") \
-    SB_NODE(multicast_group, "multicast_group") \
-    SB_NODE(mirror, "mirror") \
-    SB_NODE(meter, "meter") \
-    SB_NODE(datapath_binding, "datapath_binding") \
-    SB_NODE(port_binding, "port_binding") \
-    SB_NODE(mac_binding, "mac_binding") \
-    SB_NODE(dns, "dns") \
-    SB_NODE(ha_chassis_group, "ha_chassis_group") \
-    SB_NODE(ip_multicast, "ip_multicast") \
-    SB_NODE(igmp_group, "igmp_group") \
-    SB_NODE(service_monitor, "service_monitor") \
-    SB_NODE(load_balancer, "load_balancer") \
-    SB_NODE(bfd, "bfd") \
-    SB_NODE(fdb, "fdb") \
-    SB_NODE(static_mac_binding, "static_mac_binding") \
-    SB_NODE(chassis_template_var, "chassis_template_var") \
-    SB_NODE(logical_dp_group, "logical_dp_group") \
-    SB_NODE(ecmp_nexthop, "ecmp_nexthop") \
-    SB_NODE(acl_id, "acl_id") \
-    SB_NODE(advertised_route, "advertised_route") \
-    SB_NODE(learned_route, "learned_route")
+    SB_NODE(sb_global) \
+    SB_NODE(chassis) \
+    SB_NODE(address_set) \
+    SB_NODE(port_group) \
+    SB_NODE(logical_flow) \
+    SB_NODE(multicast_group) \
+    SB_NODE(mirror) \
+    SB_NODE(meter) \
+    SB_NODE(datapath_binding) \
+    SB_NODE(port_binding) \
+    SB_NODE(mac_binding) \
+    SB_NODE(dns) \
+    SB_NODE(ha_chassis_group) \
+    SB_NODE(ip_multicast) \
+    SB_NODE(igmp_group) \
+    SB_NODE(service_monitor) \
+    SB_NODE(load_balancer) \
+    SB_NODE(bfd) \
+    SB_NODE(fdb) \
+    SB_NODE(static_mac_binding) \
+    SB_NODE(chassis_template_var) \
+    SB_NODE(logical_dp_group) \
+    SB_NODE(ecmp_nexthop) \
+    SB_NODE(acl_id) \
+    SB_NODE(advertised_route) \
+    SB_NODE(learned_route)
 
 enum sb_engine_node {
-#define SB_NODE(NAME, NAME_STR) SB_##NAME,
+#define SB_NODE(NAME) SB_##NAME,
     SB_NODES
 #undef SB_NODE
 };
@@ -126,7 +130,7 @@ enum sb_engine_node {
  * en_sb_<TABLE_NAME>_init()
  * en_sb_<TABLE_NAME>_cleanup()
  */
-#define SB_NODE(NAME, NAME_STR) ENGINE_FUNC_SB(NAME);
+#define SB_NODE(NAME) ENGINE_FUNC_SB(NAME);
     SB_NODES
 #undef SB_NODE
 
@@ -137,62 +141,59 @@ enum sb_engine_node {
  *
  * Define nodes as static to avoid sparse errors.
  */
-#define NB_NODE(NAME, NAME_STR) static ENGINE_NODE_NB(NAME, NAME_STR);
+#define NB_NODE(NAME) static ENGINE_NODE_NB(NAME);
     NB_NODES
 #undef NB_NODE
 
-#define SB_NODE(NAME, NAME_STR) static ENGINE_NODE_SB(NAME, NAME_STR);
+#define SB_NODE(NAME) static ENGINE_NODE_SB(NAME);
     SB_NODES
 #undef SB_NODE
 
 /* Define engine nodes for other nodes. They should be defined as static to
  * avoid sparse errors. */
-static ENGINE_NODE_WITH_CLEAR_TRACK_DATA(northd, "northd");
-static ENGINE_NODE(sync_from_sb, "sync_from_sb");
-static ENGINE_NODE(sampling_app, "sampling_app");
-static ENGINE_NODE(lflow, "lflow");
-static ENGINE_NODE(mac_binding_aging, "mac_binding_aging");
-static ENGINE_NODE(mac_binding_aging_waker, "mac_binding_aging_waker");
-static ENGINE_NODE(northd_output, "northd_output");
-static ENGINE_NODE(sync_meters, "sync_meters");
-static ENGINE_NODE(sync_to_sb, "sync_to_sb");
-static ENGINE_NODE(sync_to_sb_addr_set, "sync_to_sb_addr_set");
-static ENGINE_NODE_WITH_CLEAR_TRACK_DATA(port_group, "port_group");
-static ENGINE_NODE(fdb_aging, "fdb_aging");
-static ENGINE_NODE(fdb_aging_waker, "fdb_aging_waker");
-static ENGINE_NODE(sync_to_sb_lb, "sync_to_sb_lb");
-static ENGINE_NODE(sync_to_sb_pb, "sync_to_sb_pb");
-static ENGINE_NODE_WITH_CLEAR_TRACK_DATA(global_config, "global_config");
-static ENGINE_NODE_WITH_CLEAR_TRACK_DATA(lb_data, "lb_data");
-static ENGINE_NODE_WITH_CLEAR_TRACK_DATA(lr_nat, "lr_nat");
-static ENGINE_NODE_WITH_CLEAR_TRACK_DATA(lr_stateful, "lr_stateful");
-static ENGINE_NODE_WITH_CLEAR_TRACK_DATA(ls_stateful, "ls_stateful");
-static ENGINE_NODE(route_policies, "route_policies");
-static ENGINE_NODE(routes, "routes");
-static ENGINE_NODE(bfd, "bfd");
-static ENGINE_NODE(bfd_sync, "bfd_sync");
-static ENGINE_NODE(ecmp_nexthop, "ecmp_nexthop");
-static ENGINE_NODE(multicast_igmp, "multicast_igmp");
-static ENGINE_NODE(acl_id, "acl_id");
-static ENGINE_NODE(advertised_route_sync, "advertised_route_sync");
-static ENGINE_NODE_WITH_CLEAR_TRACK_DATA(learned_route_sync,
-                                         "learned_route_sync");
-static ENGINE_NODE(dynamic_routes, "dynamic_routes");
-static ENGINE_NODE_WITH_CLEAR_TRACK_DATA(group_ecmp_route, "group_ecmp_route");
+static ENGINE_NODE(northd, CLEAR_TRACKED_DATA);
+static ENGINE_NODE(sync_from_sb);
+static ENGINE_NODE(sampling_app);
+static ENGINE_NODE(lflow);
+static ENGINE_NODE(mac_binding_aging);
+static ENGINE_NODE(mac_binding_aging_waker);
+static ENGINE_NODE(northd_output);
+static ENGINE_NODE(sync_meters);
+static ENGINE_NODE(sync_to_sb);
+static ENGINE_NODE(sync_to_sb_addr_set);
+static ENGINE_NODE(port_group, CLEAR_TRACKED_DATA);
+static ENGINE_NODE(fdb_aging);
+static ENGINE_NODE(fdb_aging_waker);
+static ENGINE_NODE(sync_to_sb_lb);
+static ENGINE_NODE(sync_to_sb_pb);
+static ENGINE_NODE(global_config, CLEAR_TRACKED_DATA);
+static ENGINE_NODE(lb_data, CLEAR_TRACKED_DATA);
+static ENGINE_NODE(lr_nat, CLEAR_TRACKED_DATA);
+static ENGINE_NODE(lr_stateful, CLEAR_TRACKED_DATA);
+static ENGINE_NODE(ls_stateful, CLEAR_TRACKED_DATA);
+static ENGINE_NODE(route_policies);
+static ENGINE_NODE(routes);
+static ENGINE_NODE(bfd);
+static ENGINE_NODE(bfd_sync);
+static ENGINE_NODE(ecmp_nexthop);
+static ENGINE_NODE(multicast_igmp);
+static ENGINE_NODE(acl_id);
+static ENGINE_NODE(advertised_route_sync);
+static ENGINE_NODE(learned_route_sync, CLEAR_TRACKED_DATA);
+static ENGINE_NODE(dynamic_routes);
+static ENGINE_NODE(group_ecmp_route, CLEAR_TRACKED_DATA);
+static ENGINE_NODE(datapath_logical_router, CLEAR_TRACKED_DATA);
+static ENGINE_NODE(datapath_logical_switch, CLEAR_TRACKED_DATA);
+static ENGINE_NODE(datapath_sync, CLEAR_TRACKED_DATA);
+static ENGINE_NODE(datapath_synced_logical_router, CLEAR_TRACKED_DATA);
+static ENGINE_NODE(datapath_synced_logical_switch, CLEAR_TRACKED_DATA);
+static ENGINE_NODE(ic_learned_svc_monitors);
 
 void inc_proc_northd_init(struct ovsdb_idl_loop *nb,
                           struct ovsdb_idl_loop *sb)
 {
     /* Define relationships between nodes where first argument is dependent
      * on the second argument */
-    engine_add_input(&en_lb_data, &en_nb_load_balancer,
-                     lb_data_load_balancer_handler);
-    engine_add_input(&en_lb_data, &en_nb_load_balancer_group,
-                     lb_data_load_balancer_group_handler);
-    engine_add_input(&en_lb_data, &en_nb_logical_switch,
-                     lb_data_logical_switch_handler);
-    engine_add_input(&en_lb_data, &en_nb_logical_router,
-                     lb_data_logical_router_handler);
 
     engine_add_input(&en_sampling_app, &en_nb_sampling_app, NULL);
 
@@ -209,20 +210,55 @@ void inc_proc_northd_init(struct ovsdb_idl_loop *nb,
     engine_add_input(&en_acl_id, &en_nb_acl, NULL);
     engine_add_input(&en_acl_id, &en_sb_acl_id, NULL);
 
+    engine_add_input(&en_datapath_logical_switch, &en_nb_logical_switch,
+                     datapath_logical_switch_handler);
+    engine_add_input(&en_datapath_logical_switch, &en_global_config,
+                     datapath_logical_switch_handler);
+
+    engine_add_input(&en_datapath_logical_router, &en_nb_logical_router,
+                     en_datapath_logical_router_logical_router_handler);
+
+    engine_add_input(&en_datapath_sync, &en_global_config,
+                     datapath_sync_global_config_handler);
+    engine_add_input(&en_datapath_sync, &en_sb_datapath_binding,
+                     datapath_sync_sb_datapath_binding);
+    engine_add_input(&en_datapath_sync, &en_datapath_logical_switch,
+                     datapath_sync_logical_switch_handler);
+    engine_add_input(&en_datapath_sync, &en_datapath_logical_router,
+                     datapath_sync_logical_router_handler);
+
+    engine_add_input(&en_datapath_synced_logical_router, &en_datapath_sync,
+                     en_datapath_synced_logical_router_datapath_sync_handler);
+    engine_add_input(&en_datapath_synced_logical_switch, &en_datapath_sync,
+                     en_datapath_synced_logical_switch_datapath_sync_handler);
+
+    engine_add_input(&en_lb_data, &en_nb_load_balancer,
+                     lb_data_load_balancer_handler);
+    engine_add_input(&en_lb_data, &en_nb_load_balancer_group,
+                     lb_data_load_balancer_group_handler);
+    engine_add_input(&en_lb_data, &en_datapath_synced_logical_switch,
+                     lb_data_synced_logical_switch_handler);
+    engine_add_input(&en_lb_data, &en_datapath_synced_logical_router,
+                     lb_data_synced_logical_router_handler);
+
+    engine_add_input(&en_ic_learned_svc_monitors,
+                     &en_sb_service_monitor, NULL);
+
     engine_add_input(&en_northd, &en_nb_mirror, NULL);
+    engine_add_input(&en_northd, &en_nb_mirror_rule, NULL);
     engine_add_input(&en_northd, &en_nb_static_mac_binding, NULL);
     engine_add_input(&en_northd, &en_nb_chassis_template_var, NULL);
 
     engine_add_input(&en_northd, &en_sb_chassis, NULL);
     engine_add_input(&en_northd, &en_sb_mirror, NULL);
     engine_add_input(&en_northd, &en_sb_meter, NULL);
-    engine_add_input(&en_northd, &en_sb_datapath_binding, NULL);
     engine_add_input(&en_northd, &en_sb_dns, NULL);
     engine_add_input(&en_northd, &en_sb_ha_chassis_group, NULL);
     engine_add_input(&en_northd, &en_sb_ip_multicast, NULL);
     engine_add_input(&en_northd, &en_sb_service_monitor, NULL);
     engine_add_input(&en_northd, &en_sb_static_mac_binding, NULL);
     engine_add_input(&en_northd, &en_sb_chassis_template_var, NULL);
+    engine_add_input(&en_northd, &en_ic_learned_svc_monitors, NULL);
     engine_add_input(&en_northd, &en_sb_fdb, northd_sb_fdb_change_handler);
     engine_add_input(&en_northd, &en_global_config,
                      northd_global_config_handler);
@@ -240,11 +276,13 @@ void inc_proc_northd_init(struct ovsdb_idl_loop *nb,
 
     engine_add_input(&en_northd, &en_sb_port_binding,
                      northd_sb_port_binding_handler);
-    engine_add_input(&en_northd, &en_nb_logical_switch,
+    engine_add_input(&en_northd, &en_datapath_synced_logical_switch,
                      northd_nb_logical_switch_handler);
-    engine_add_input(&en_northd, &en_nb_logical_router,
+    engine_add_input(&en_northd, &en_datapath_synced_logical_router,
                      northd_nb_logical_router_handler);
     engine_add_input(&en_northd, &en_lb_data, northd_lb_data_handler);
+    engine_add_input(&en_northd, &en_nb_port_group,
+                     northd_nb_port_group_handler);
 
     engine_add_input(&en_lr_nat, &en_northd, lr_nat_northd_handler);
 
@@ -256,6 +294,7 @@ void inc_proc_northd_init(struct ovsdb_idl_loop *nb,
     engine_add_input(&en_ls_stateful, &en_northd, ls_stateful_northd_handler);
     engine_add_input(&en_ls_stateful, &en_port_group,
                      ls_stateful_port_group_handler);
+    engine_add_input(&en_ls_stateful, &en_nb_acl, ls_stateful_acl_handler);
 
     engine_add_input(&en_mac_binding_aging, &en_sb_mac_binding, NULL);
     engine_add_input(&en_mac_binding_aging, &en_northd, NULL);
@@ -318,7 +357,7 @@ void inc_proc_northd_init(struct ovsdb_idl_loop *nb,
     engine_add_input(&en_group_ecmp_route, &en_learned_route_sync,
                      group_ecmp_route_learned_route_change_handler);
 
-    engine_add_input(&en_sync_meters, &en_nb_acl, NULL);
+    engine_add_input(&en_sync_meters, &en_nb_acl, sync_meters_nb_acl_handler);
     engine_add_input(&en_sync_meters, &en_nb_meter, NULL);
     engine_add_input(&en_sync_meters, &en_sb_meter, NULL);
 
@@ -327,7 +366,6 @@ void inc_proc_northd_init(struct ovsdb_idl_loop *nb,
     engine_add_input(&en_multicast_igmp, &en_sb_multicast_group, NULL);
     engine_add_input(&en_multicast_igmp, &en_sb_igmp_group, NULL);
 
-    engine_add_input(&en_lflow, &en_nb_acl, NULL);
     engine_add_input(&en_lflow, &en_sync_meters, NULL);
     engine_add_input(&en_lflow, &en_sb_logical_flow, NULL);
     engine_add_input(&en_lflow, &en_sb_multicast_group, NULL);
@@ -345,12 +383,18 @@ void inc_proc_northd_init(struct ovsdb_idl_loop *nb,
     engine_add_input(&en_lflow, &en_sampling_app, NULL);
 
     engine_add_input(&en_lflow, &en_northd, lflow_northd_handler);
-    engine_add_input(&en_lflow, &en_port_group, lflow_port_group_handler);
+    /* No need for an explicit handler for port_groups in the en_lflow node.
+     * Stateful configuration changes are passed through the en_ls_stateful
+     * input dependancy. Still needs access to en_port_port_group (input)
+     * data to look up port groups.  */
+    engine_add_input(&en_lflow, &en_port_group, engine_noop_handler);
     engine_add_input(&en_lflow, &en_lr_stateful, lflow_lr_stateful_handler);
     engine_add_input(&en_lflow, &en_ls_stateful, lflow_ls_stateful_handler);
     engine_add_input(&en_lflow, &en_multicast_igmp,
                      lflow_multicast_igmp_handler);
     engine_add_input(&en_lflow, &en_sb_acl_id, NULL);
+    engine_add_input(&en_lflow, &en_ic_learned_svc_monitors,
+                     lflow_ic_learned_svc_mons_handler);
 
     engine_add_input(&en_sync_to_sb_addr_set, &en_northd, NULL);
     engine_add_input(&en_sync_to_sb_addr_set, &en_lr_stateful, NULL);
@@ -490,12 +534,27 @@ void inc_proc_northd_init(struct ovsdb_idl_loop *nb,
     engine_ovsdb_node_add_index(&en_sb_port_binding,
                                 "sbrec_port_binding_by_name",
                                 sbrec_port_binding_by_name);
+
+    struct ovsdb_idl_index *nbrec_mirror_by_type_and_sink
+        = ovsdb_idl_index_create2(nb->idl, &nbrec_mirror_col_type,
+                                  &nbrec_mirror_col_sink);
+    engine_ovsdb_node_add_index(&en_nb_mirror,
+                                "nbrec_mirror_by_type_and_sink",
+                                nbrec_mirror_by_type_and_sink);
+
     struct ovsdb_idl_index *sbrec_ecmp_nexthop_by_ip_and_port
         = ecmp_nexthop_index_create(sb->idl);
     engine_ovsdb_node_add_index(&en_sb_ecmp_nexthop,
                                 "sbrec_ecmp_nexthop_by_ip_and_port",
                                 sbrec_ecmp_nexthop_by_ip_and_port);
 
+    struct ovsdb_idl_index *sbrec_service_monitor_by_learned_type
+        = ovsdb_idl_index_create1(sb->idl,
+                                  &sbrec_service_monitor_col_ic_learned);
+    engine_ovsdb_node_add_index(&en_sb_service_monitor,
+                                "sbrec_service_monitor_by_learned_type",
+                                sbrec_service_monitor_by_learned_type);
+
     struct ed_type_global_config *global_config =
         engine_get_internal_data(&en_global_config);
     unixctl_command_register("debug/chassis-features-list", "", 0, 0,
