@@ -93,13 +93,21 @@ void ovn_northd_lb_vip_init(struct ovn_northd_lb_vip *lb_vip_nb,
                             bool template)
 {
     lb_vip_nb->backend_ips = xstrdup(backend_ips);
-    lb_vip_nb->n_backends = lb_vip->n_backends;
+    lb_vip_nb->n_backends = vector_len(&lb_vip->backends);
     lb_vip_nb->backends_nb = xcalloc(lb_vip_nb->n_backends,
                                      sizeof *lb_vip_nb->backends_nb);
     lb_vip_nb->lb_health_check =
         ovn_lb_get_health_check(nbrec_lb, vip_port_str, template);
 }
 
+/*
+ * Initializes health check configuration for load balancer VIP
+ * backends. Parses the ip_port_mappings in the format :
+ * "ip:logical_port:src_ip[:az_name]".
+ * If az_name is present and non-empty, it indicates this is a
+ * remote service monitor (backend is in another availability zone),
+ * it should be propogated to another AZ by interconnection processing.
+ */
 static void
 ovn_lb_vip_backends_health_check_init(const struct ovn_northd_lb *lb,
                                       const struct ovn_lb_vip *lb_vip,
@@ -107,8 +115,9 @@ ovn_lb_vip_backends_health_check_init(const struct ovn_northd_lb *lb,
 {
     struct ds key = DS_EMPTY_INITIALIZER;
 
-    for (size_t j = 0; j < lb_vip->n_backends; j++) {
-        struct ovn_lb_backend *backend = &lb_vip->backends[j];
+    for (size_t j = 0; j < vector_len(&lb_vip->backends); j++) {
+        const struct ovn_lb_backend *backend =
+            vector_get_ptr(&lb_vip->backends, j);
         ds_clear(&key);
         ds_put_format(&key, IN6_IS_ADDR_V4MAPPED(&lb_vip->vip)
                       ? "%s" : "[%s]", backend->ip_str);
@@ -119,22 +128,64 @@ ovn_lb_vip_backends_health_check_init(const struct ovn_northd_lb *lb,
         }
 
         char *svc_mon_src_ip = NULL;
+        char *az_name = NULL;
+        bool is_remote = false;
         char *port_name = xstrdup(s);
-        char *p = strstr(port_name, ":");
-        if (p) {
-            *p = 0;
-            p++;
-            struct sockaddr_storage svc_mon_src_addr;
-            if (!inet_parse_address(p, &svc_mon_src_addr)) {
-                static struct vlog_rate_limit rl =
-                    VLOG_RATE_LIMIT_INIT(5, 1);
-                VLOG_WARN_RL(&rl, "Invalid svc mon src IP %s", p);
-            } else {
-                struct ds src_ip_s = DS_EMPTY_INITIALIZER;
-                ss_format_address_nobracks(&svc_mon_src_addr,
-                                            &src_ip_s);
-                svc_mon_src_ip = ds_steal_cstr(&src_ip_s);
+        char *src_ip = NULL;
+
+        char *first_colon = strchr(port_name, ':');
+        if (!first_colon) {
+            free(port_name);
+            continue;
+        }
+        *first_colon = '\0';
+
+        if (first_colon[1] == '[') {
+            /* IPv6 case - format: port:[ipv6]:az or port:[ipv6] */
+            char *ip_end = strchr(first_colon + 2, ']');
+            if (!ip_end) {
+                VLOG_WARN("Malformed IPv6 address in backend %s", s);
+                free(port_name);
+                continue;
             }
+
+            src_ip = first_colon + 2;
+            *ip_end = '\0';
+
+            if (ip_end[1] == ':') {
+                az_name = ip_end + 2;
+                if (!*az_name) {
+                    VLOG_WARN("Empty AZ name specified for backend %s",
+                              port_name);
+                    free(port_name);
+                    continue;
+                }
+                is_remote = true;
+            }
+        } else {
+            /* IPv4 case - format: port:ip:az or port:ip */
+            src_ip = first_colon + 1;
+            char *az_colon = strchr(src_ip, ':');
+            if (az_colon) {
+                *az_colon = '\0';
+                az_name = az_colon + 1;
+                if (!*az_name) {
+                    VLOG_WARN("Empty AZ name specified for backend %s",
+                              port_name);
+                    free(port_name);
+                    continue;
+                }
+            is_remote = true;
+            }
+        }
+
+        struct sockaddr_storage svc_mon_src_addr;
+        if (!src_ip || !inet_parse_address(src_ip, &svc_mon_src_addr)) {
+            VLOG_WARN("Invalid svc mon src IP %s", src_ip ? src_ip : "NULL");
+        } else {
+            struct ds src_ip_s = DS_EMPTY_INITIALIZER;
+            ss_format_address_nobracks(&svc_mon_src_addr, &src_ip_s);
+            svc_mon_src_ip = ds_steal_cstr(&src_ip_s);
         }
 
         if (svc_mon_src_ip) {
@@ -143,6 +194,8 @@ ovn_lb_vip_backends_health_check_init(const struct ovn_northd_lb *lb,
             backend_nb->health_check = true;
             backend_nb->logical_port = xstrdup(port_name);
             backend_nb->svc_mon_src_ip = svc_mon_src_ip;
+            backend_nb->az_name = is_remote ? xstrdup(az_name) : NULL;
+            backend_nb->remote_backend = is_remote;
         }
         free(port_name);
     }
@@ -157,6 +210,7 @@ void ovn_northd_lb_vip_destroy(struct ovn_northd_lb_vip *vip)
     for (size_t i = 0; i < vip->n_backends; i++) {
         free(vip->backends_nb[i].logical_port);
         free(vip->backends_nb[i].svc_mon_src_ip);
+        free(vip->backends_nb[i].az_name);
     }
     free(vip->backends_nb);
 }
@@ -256,6 +310,14 @@ ovn_lb_get_health_check(const struct nbrec_load_balancer *nbrec_lb,
     return NULL;
 }
 
+static bool
+validate_snap_ip_address(const char *snat_ip)
+{
+    ovs_be32 ip;
+
+    return ip_parse(snat_ip, &ip);
+}
+
 static void
 ovn_northd_lb_init(struct ovn_northd_lb *lb,
                    const struct nbrec_load_balancer *nbrec_lb)
@@ -296,6 +358,12 @@ ovn_northd_lb_init(struct ovn_northd_lb *lb,
     }
     lb->affinity_timeout = affinity_timeout;
 
+    const char *snat_ip = smap_get(&nbrec_lb->options, "hairpin_snat_ip");
+
+    if (snat_ip && validate_snap_ip_address(snat_ip)) {
+        lb->hairpin_snat_ip = xstrdup(snat_ip);
+    }
+
     sset_init(&lb->ips_v4);
     sset_init(&lb->ips_v6);
     struct smap_node *node;
@@ -412,6 +480,7 @@ ovn_northd_lb_cleanup(struct ovn_northd_lb *lb)
     sset_destroy(&lb->ips_v4);
     sset_destroy(&lb->ips_v6);
     free(lb->selection_fields);
+    free(lb->hairpin_snat_ip);
     lb->selection_fields = NULL;
     lb->health_checks = false;
 }
@@ -565,7 +634,7 @@ ovn_lb_datapaths_create(const struct ovn_northd_lb *lb, size_t n_ls_datapaths,
     lb_dps->nb_ls_map = bitmap_allocate(n_ls_datapaths);
     lb_dps->nb_lr_map = bitmap_allocate(n_lr_datapaths);
     lb_dps->lflow_ref = lflow_ref_create();
-
+    hmapx_init(&lb_dps->ls_lb_with_stateless_mode);
     return lb_dps;
 }
 
@@ -575,6 +644,7 @@ ovn_lb_datapaths_destroy(struct ovn_lb_datapaths *lb_dps)
     bitmap_free(lb_dps->nb_lr_map);
     bitmap_free(lb_dps->nb_ls_map);
     lflow_ref_destroy(lb_dps->lflow_ref);
+    hmapx_destroy(&lb_dps->ls_lb_with_stateless_mode);
     free(lb_dps);
 }
 
