@@ -726,18 +726,6 @@ lflow_table_add_lflow(struct lflow_table *lflow_table,
     lflow_hash_unlock(hash_lock);
 }
 
-void
-lflow_table_add_lflow_default_drop(struct lflow_table *lflow_table,
-                                   const struct ovn_datapath *od,
-                                   enum ovn_stage stage,
-                                   const char *where,
-                                   struct lflow_ref *lflow_ref)
-{
-    lflow_table_add_lflow(lflow_table, od, NULL, 0, stage, 0, "1",
-                          debug_drop_action(), NULL, NULL, NULL,
-                          where, NULL, lflow_ref);
-}
-
 struct ovn_dp_group *
 ovn_dp_group_get(struct hmap *dp_groups, size_t desired_n,
                  const unsigned long *desired_bitmap,
@@ -1130,7 +1118,7 @@ sync_lflow_to_sb(struct ovn_lflow *lflow,
     }
 
     if (lflow->od) {
-        sbrec_logical_flow_set_logical_datapath(sbflow, lflow->od->sb);
+        sbrec_logical_flow_set_logical_datapath(sbflow, lflow->od->sdp->sb_dp);
         sbrec_logical_flow_set_logical_dp_group(sbflow, NULL);
     } else {
         sbrec_logical_flow_set_logical_datapath(sbflow, NULL);
@@ -1238,7 +1226,7 @@ ovn_sb_insert_or_update_logical_dp_group(
 
     sb = xmalloc(bitmap_count1(dpg_bitmap, ods_size(datapaths)) * sizeof *sb);
     BITMAP_FOR_EACH_1 (index, ods_size(datapaths), dpg_bitmap) {
-        sb[n++] = datapaths->array[index]->sb;
+        sb[n++] = datapaths->array[index]->sdp->sb_dp;
     }
     if (!dp_group) {
         struct uuid dpg_uuid = uuid_random();
