@@ -532,49 +532,48 @@ update_sequence_numbers(int64_t loop_start_time,
 
     /* Update northbound sb_cfg if appropriate. */
     int64_t sb_cfg = sb_loop->cur_cfg;
-    if (nb && sb_cfg && nb->sb_cfg != sb_cfg) {
+    if (sb_cfg && nb->sb_cfg != sb_cfg) {
         nbrec_nb_global_set_sb_cfg(nb, sb_cfg);
         nbrec_nb_global_set_sb_cfg_timestamp(nb, loop_start_time);
     }
 
     /* Update northbound hv_cfg if appropriate. */
-    if (nb) {
-        /* Find minimum nb_cfg among all chassis. */
-        const struct sbrec_chassis_private *chassis_priv;
-        int64_t hv_cfg = nb->nb_cfg;
-        int64_t hv_cfg_ts = 0;
-        SBREC_CHASSIS_PRIVATE_FOR_EACH (chassis_priv, ovnsb_idl) {
-            const struct sbrec_chassis *chassis = chassis_priv->chassis;
-            if (chassis) {
-                if (smap_get_bool(&chassis->other_config,
-                                  "is-remote", false)) {
-                    /* Skip remote chassises. */
-                    continue;
-                }
-            } else {
-                static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(1, 1);
-                VLOG_WARN_RL(&rl, "Chassis does not exist for "
-                             "Chassis_Private record, name: %s",
-                             chassis_priv->name);
-            }
 
-            /* Detect if overflows happened within the cfg update. */
-            int64_t delta = chassis_priv->nb_cfg - hv_cfg;
-            if (chassis_priv->nb_cfg < hv_cfg || delta > INT32_MAX) {
-                hv_cfg = chassis_priv->nb_cfg;
-                hv_cfg_ts = chassis_priv->nb_cfg_timestamp;
-            } else if (chassis_priv->nb_cfg == hv_cfg &&
-                       chassis_priv->nb_cfg_timestamp > hv_cfg_ts) {
-                hv_cfg_ts = chassis_priv->nb_cfg_timestamp;
+    /* Find minimum nb_cfg among all chassis. */
+    const struct sbrec_chassis_private *chassis_priv;
+    int64_t hv_cfg = nb->nb_cfg;
+    int64_t hv_cfg_ts = 0;
+    SBREC_CHASSIS_PRIVATE_FOR_EACH (chassis_priv, ovnsb_idl) {
+        const struct sbrec_chassis *chassis = chassis_priv->chassis;
+        if (chassis) {
+            if (smap_get_bool(&chassis->other_config,
+                              "is-remote", false)) {
+                /* Skip remote chassises. */
+                continue;
             }
+        } else {
+            static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(1, 1);
+            VLOG_WARN_RL(&rl, "Chassis does not exist for "
+                         "Chassis_Private record, name: %s",
+                         chassis_priv->name);
         }
 
-        /* Update hv_cfg. */
-        if (nb->hv_cfg != hv_cfg) {
-            nbrec_nb_global_set_hv_cfg(nb, hv_cfg);
-            nbrec_nb_global_set_hv_cfg_timestamp(nb, hv_cfg_ts);
+        /* Detect if overflows happened within the cfg update. */
+        int64_t delta = chassis_priv->nb_cfg - hv_cfg;
+        if (chassis_priv->nb_cfg < hv_cfg || delta > INT32_MAX) {
+            hv_cfg = chassis_priv->nb_cfg;
+            hv_cfg_ts = chassis_priv->nb_cfg_timestamp;
+        } else if (chassis_priv->nb_cfg == hv_cfg &&
+                   chassis_priv->nb_cfg_timestamp > hv_cfg_ts) {
+            hv_cfg_ts = chassis_priv->nb_cfg_timestamp;
         }
     }
+
+    /* Update hv_cfg. */
+    if (nb->hv_cfg != hv_cfg) {
+        nbrec_nb_global_set_hv_cfg(nb, hv_cfg);
+        nbrec_nb_global_set_hv_cfg_timestamp(nb, hv_cfg_ts);
+    }
 }
 
 static void
@@ -798,6 +797,15 @@ run_memory_trimmer(struct ovsdb_idl *ovnnb_idl, bool activity)
     memory_trimmer_wait(mt);
 }
 
+static bool
+sb_cfg_is_in_sync(struct ovsdb_idl *ovnnb_idl,
+                  struct ovsdb_idl_loop *sb_loop)
+{
+    const struct nbrec_nb_global *nb = nbrec_nb_global_first(ovnnb_idl);
+
+    return !(nb && sb_loop->cur_cfg && nb->sb_cfg != sb_loop->cur_cfg);
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -954,6 +962,10 @@ main(int argc, char *argv[])
         ovsdb_idl_omit_alert(ovnsb_idl_loop.idl,
                              &sbrec_advertised_route_columns[i]);
     }
+    for (size_t i = 0; i < SBREC_STATIC_MAC_BINDING_N_COLUMNS; i++) {
+        ovsdb_idl_omit_alert(ovnsb_idl_loop.idl,
+                             &sbrec_static_mac_binding_columns[i]);
+    }
 
     unixctl_command_register("sb-connection-status", "", 0, 0,
                              ovn_conn_show, ovnsb_idl_loop.idl);
@@ -1065,8 +1077,17 @@ main(int argc, char *argv[])
                 if (ovnnb_txn && ovnsb_txn &&
                     inc_proc_northd_can_run(&eng_ctx)) {
                     int64_t loop_start_time = time_wall_msec();
-                    activity = inc_proc_northd_run(ovnnb_txn, ovnsb_txn,
-                                                   &eng_ctx);
+
+                    if (sb_cfg_is_in_sync(ovnnb_idl_loop.idl,
+                                          &ovnsb_idl_loop)) {
+                        activity = inc_proc_northd_run(ovnnb_txn,
+                                                       ovnsb_txn,
+                                                       &eng_ctx);
+                    } else {
+                        poll_immediate_wake();
+                        clear_idl_track = false;
+                    }
+
                     check_and_add_supported_dhcp_opts_to_sb_db(
                                  ovnsb_txn, ovnsb_idl_loop.idl);
                     check_and_add_supported_dhcpv6_opts_to_sb_db(
