@@ -954,7 +954,8 @@ test_tree_shape_exhaustively(struct expr *expr, struct shash *symtab,
                 test_rule = xmalloc(sizeof *test_rule);
                 cls_rule_init(&test_rule->cr, &m->match, 0);
                 classifier_insert(&cls, &test_rule->cr, OVS_VERSION_MIN,
-                                  m->conjunctions, m->n);
+                                  vector_get_array(&m->conjunctions),
+                                  vector_len(&m->conjunctions));
             }
         }
         for (int subst = 0; subst < 1 << (n_bits * n_nvars + n_svars);
@@ -1341,7 +1342,8 @@ test_parse_actions(struct ovs_cmdl_context *ctx OVS_UNUSED)
             .dhcpv6_opts = &dhcpv6_opts,
             .nd_ra_opts = &nd_ra_opts,
             .controller_event_opts = &event_opts,
-            .n_tables = 24,
+            .pipeline = OVNACT_P_INGRESS,
+            .n_tables = LOG_PIPELINE_INGRESS_LEN,
             .cur_ltable = 10,
         };
 
@@ -1378,6 +1380,7 @@ test_parse_actions(struct ovs_cmdl_context *ctx OVS_UNUSED)
                 .lb_hairpin_reply_ptable = OFTABLE_CHK_LB_HAIRPIN_REPLY,
                 .ct_snat_vip_ptable = OFTABLE_CT_SNAT_HAIRPIN,
                 .fdb_ptable = OFTABLE_GET_FDB,
+                .remote_fdb_ptable = OFTABLE_GET_REMOTE_FDB,
                 .fdb_lookup_ptable = OFTABLE_LOOKUP_FDB,
                 .common_nat_ct_zone = MFF_LOG_DNAT_ZONE,
                 .in_port_sec_ptable = OFTABLE_CHK_IN_PORT_SEC,
@@ -1386,6 +1389,7 @@ test_parse_actions(struct ovs_cmdl_context *ctx OVS_UNUSED)
                 .ct_nw_dst_load_table = OFTABLE_CT_ORIG_NW_DST_LOAD,
                 .ct_ip6_dst_load_table = OFTABLE_CT_ORIG_IP6_DST_LOAD,
                 .ct_tp_dst_load_table = OFTABLE_CT_ORIG_TP_DST_LOAD,
+                .ct_proto_load_table = OFTABLE_CT_ORIG_PROTO_LOAD,
                 .flood_remote_table = OFTABLE_FLOOD_REMOTE_CHASSIS,
                 .ct_state_save_table = OFTABLE_CT_STATE_SAVE,
                 .lflow_uuid.parts =
