@@ -48,7 +48,7 @@ static bool oneline;
 /* --dry-run: Do not commit any changes. */
 static bool dry_run;
 
-/* SSL options */
+/* SSL/TLS options. */
 static const char *ssl_private_key_file;
 static const char *ssl_certificate_file;
 static const char *ssl_ca_cert_file;
@@ -618,6 +618,10 @@ apply_options_direct(const struct ovn_dbctl_options *dbctl_options,
             stream_ssl_set_ciphers(optarg);
             break;
 
+        case OPT_SSL_CIPHERSUITES:
+            stream_ssl_set_ciphersuites(optarg);
+            break;
+
         case OPT_BOOTSTRAP_CA_CERT:
             stream_ssl_set_ca_cert_file(po->arg, true);
             break;
